import numpy as np
import matplotlib.pyplot as plt
import DF
import DFset

def PlotAllDFs(listDFs,hardcopy=False,title='',linetype='',coinciding=False,nolegendintra=False,figsize=(10,7),dpi=80):    
    '''Plot distribution functions grouped by their order in a input list of DFsets.
    Hardcopy - create eps-copy of the plot,    
    linetype - define type of line using same syntax as matplotlib.pyplot
    coinciding - Plot only coinciding functions (by Atom Types, Mol)
    nolegendintra - turn off legend on plots with intramolecular DFs. Usually it is useful then legend overlaps with lines on plot. 
    figsize, dpi - sizeand resolution of the resulting plot in inches and dpi '''
# internal supplementary plotting procedure    
    def _plot_(DFs,title='', figsize=(10,7), dpi=80, linetype='',nolegendintra=False,hardcopy=False, legend=None):
        fig=plt.figure(figsize=figsize,dpi=dpi)        
        plt.hold(False)                
        for i in DFs:
            if legend!=None:
                lgnd=legend[DFs.index(i)]
            else:
                lgnd=None
            if linetype=='':
                i.Plot(hidden=True,nolegendintra=nolegendintra,noshow=True,legend=lgnd) 
            else:
                i.Plot(hidden=True,linetype=linetype,nolegendintra=nolegendintra, noshow=True, legend=lgnd)
            plt.hold(True)            
        plt.title(title)
        plt.xlabel('r, A')
        plt.legend(loc=0)
        if hardcopy:
             if title[0]=='.':
                 title=title[1:]
             plt.savefig(title.replace(' ','.')+'.eps')
        plt.show()
#-------------------------
# Check what kind of objects we have in the list:
    if isinstance(listDFs, DFset.DFset): # we deal with a single DFSet object
        print 'NOTE: Single DFset object given, so every plot will only has a single curve!'        
        listDFs=[listDFs]
# First we check if all the lists given in the input are of the same length

    maxlen=max(len(DFs) for DFs in listDFs)
    if not coinciding:        
        for DFs in listDFs:
            if len(DFs)!=maxlen:
                print "Error: Given DFs lists are of different length!"            
                return()
    # Now we plot all groups of DFs, one by one:
        for i in range(0,maxlen):
            lst=[]
            for j in range(0,len(listDFs)):
                lst.append(listDFs[j][i])
            _plot_(lst,title=title+'.{0}.{1}.{2}'.format(lst[0].Name,lst[0].Kind,lst[0].Type),
                            hardcopy=hardcopy,linetype=linetype,nolegendintra=nolegendintra,figsize=figsize,dpi=dpi)
    else:
        minlen=min(len(DFs) for DFs in listDFs)
        minset=listDFs[[len(DFs) for DFs in listDFs].index(minlen)] # shortest set of DFs
        for DF in minset.DFs_NB: # Loop over all NB-functions in the smallest set
            dfAT=set(DF.AtomTypes)
            lst=[DF for DFs in listDFs for DF in DFs.DFs_NB if set(DF.AtomTypes)==dfAT]
            _plot_(lst,title=title+'.{0}.{1}.{2}'.format(lst[0].Name,lst[0].Kind,lst[0].Type),
                            hardcopy=hardcopy,linetype=linetype,nolegendintra=nolegendintra,
                            legend=[DFs.Name for DFs in listDFs],
                            figsize=figsize,dpi=dpi)            

        pass

def AnalyzeIMCOuput(filename,DFType='RDF',iters=None,hardcopy=True,mcmfile=None,PairNamesList=None):
    '''Analyze output file <filename> produced by MagiC software and plot resulting DFs of interest
    DFType - defines function of interest: 'RDF': radial (and angle) distribution functions obtained in inverse MCcalculation; 'RDFref': reference RDFs(ADFs); 'Pot': Potential used in current iteration; 'PotNew': potential generated in current iteration; 'PotCorr': Potential correction applied at the iteration.
    iters - tuple of iterations to extract from the file. If nothing mentioned all iterations will be extracted.  Example: iters=(1,2,3) or iters=(1)
    basename - prefix given to all the DFs readed from the file    
    PairNamesList - List of pairs of atomic names to search in output file. It consists of three sublists: first of them refer to non-bonded pair interactions, the second refers to bonded pair interactions, and the third refers to bending angle (1-3) bond interactions
    E.g. PairNamesList=[['N-N','N-P','N-CH','N-CO','P-P','P-CH','P-CO','CH-CH','CH-CO','CO-CO'], ['N-P','P-C1','C2-C3','C1-C2','C1-C5'], ['N-C1-P','P-C2-C1','C1-C3-C2','C2-C4-C3']]        
    '''

    DFs=ReadMagiC(filename,iters=iters,DFType=DFType,PairNamesList=PairNamesList,mcmfile=mcmfile)
    if not isinstance(DFs, type([])):
        DFs=[DFs]
    if DFType.upper()=='RDF':            
        DFsRef=ReadMagiC(filename,iters=iters,DFType='RDFref',PairNamesList=PairNamesList,mcmfile=mcmfile)                
        DFs=DFs+[DFsRef]
    PlotAllDFs(DFs, hardcopy=hardcopy)


def DumpDFs(DFs,filename):
    '''Dump the set/list of distribution functions DFs to the file <filename> using module "pickle".'''
    import pickle
    f=open(filename,'ab')
    pickle.dump(DFs,f)
    f.close()
    
def LoadDFs(filename):
    '''Load a set/list of distribution functions DFs from the file <filename> using module "pickle".'''
    import pickle
    f=open(filename,'rb')
    DFs=pickle.load(f)
    f.close()
    return DFs
    
def SaveDFsAsText(DFs):
    '''Save every function-table from the set of DFs into a separate text-file'''    
    if isinstance(DFs,DFset.DFset) or isinstance(DFs,type([])):
        for i in DFs:
            SaveDFsAsText(i)
    else:
        DFs.Save()
        
def __LoadCharges(pots, mcmfile):
    for pot in pots: # loop over all potentials 
            if pot.Type=='NB':
                if (pot.qq == None): 
                    print '\n{}'.format(pot.Name)
                    print 'The charges of AtomTypes involved in the potential are unknown.'
                    if mcmfile==None:                    
                        print 'Please specify according mcm files in mcmfile-parameter to read the charges'
                        return
                    else:
                        print'Reading charges from mcm-files'
                        pot.GetCharges(mcmfile=mcmfile)
    return
    
def GetOptEpsilon(pots,eps_old,r1,eps_min=0,eps_max=0,npoints=100,mcmfile=None):
    '''
    Calculate optimal value of dielectric permittivity, providing fastest decay of short-range intermolecular potentials.
    Pots - set of potentials
    Eps_old is a dielectric permittivity used for inverse calculation (IMC/IBI)
    R1 - range where potential tail starts.
    eps_min,eps_max - define range of values for eps search, by default eps_min=0, eps_max=2*eps_old
    npoints - how many points in the eps_mesh to have
    mcmfile - molecular topology files, required to read charges of atomic types
    '''    
    # Generate mesh of epsilon values:
    weight=1e25
    eps_opt=0
    if (eps_min==0) and(eps_max==0):
        eps_max=eps_old*2
    eps=np.linspace(eps_min,eps_max,npoints)

# Preload charges
    __LoadCharges(pots, mcmfile)
                    
    for e in [e for e in eps if e!=0.0]: #every particular value of the eps-mesh
        w=0.0
        for pot in pots: # loop over all potentials 
            if pot.Type=='NB':
                if pot.qq == None:                    
                    if mcmfile==None:
                        print 'The charges of AtomTypes involved in the potential are unknown.'
                        print'Please specify according mcm files in mcmfile-parameter to read the charges'
                        return
                    else:
                        print'Reading charges'
                        pot.GetCharges(mcmfile=mcmfile)
                w+=pot.GetWeight(e,float(eps_old),r1)                
        if w<weight:
            weight=w
            eps_opt=e
    return eps_opt
    
def PotsEpsCorrection(pots,eps_old,eps_new,mcmfile=None):
    '''
    Create new set of short-range potentials with corrected value of dielectric permittivity for all intermolecular potentials in the given DFset "pots". Intramolecular potentials are kept untouched
    '''
    import copy
    # Preload charge     
    __LoadCharges(pots, mcmfile)                   
    newpots=copy.deepcopy(pots)    
    for pot in newpots.DFs_NB:
            U_sr_new=pot.g[:,1]+((pot.qq*1.602e-19*1.602e-19*6.022e23)/(4.0*3.1416*8.85e-12*1.0e-10*1.0e3))*(1.0/eps_old-1.0/eps_new)*(1.0/pot.g[:,0])
            pot.g[:,1]=U_sr_new
            pot.Name=pot.Name+' eps_old:%5.3f eps_new: %5.3f' %(eps_old,eps_new)
    return newpots

def TotalPots(pots,eps, mcmfile=None):
    '''
    Create a set of total potentials by including electrostatics (eps) to short-range intermolecular potentials (pots)    
    '''
    import copy       
    if isinstance(pots[0],type([])):
        TotalPots(pots[0],eps)
    else:
        newpots=copy.deepcopy(pots)
        __LoadCharges(newpots, mcmfile)
        for pot in newpots:
            if pot.Type=='NB':
                U_sr_new=pot.g[:,1]+((pot.qq*1.602e-19*1.602e-19*6.022e23)/(4.0*3.1416*8.85e-12*1.0e-10*1.0e3))*(1.0/eps)*(1.0/pot.g[:,0])
                pot.g[:,1]=U_sr_new
                pot.Name=pot.Name+' Total with eps: %5.3f' %(eps)
        return newpots

def PotsPressCorr(pots,U_corr0):
    '''
    Create new set of short-range potentials by adding a decaying linear term, which suppose to improve pressure of represented system. Intramolecular potentials are kept untouched
    Correction term is linear and has value of U_corr0 at point r=0, and value of 0 at r=rmax
    '''
    import copy
    newpots=copy.deepcopy(pots)
    print "Performing linear correction of the potentials: Ucorr(0)=",str(U_corr0)
    for pot in newpots:
        pot.PotPressCorr(U_corr0)
    return newpots
    
def PotsExport2Gromacs(pots,npoints=2500, Umax=6000,Rmaxtable=2.5,PHImaxtable=180, filename='',noplot=False,hardcopy=True,figsize=(14,7.5),dpi=120, sigma=0.5, zeroforce=True, interpol=True,ofilename=''):
    '''
    Export the given set of potentials  (pots) to a Gromacs xvg format:        
    '''
    for pot in pots:        
        if isinstance(pot,DF.Pot_NB):
            ofilename='table_{0}_{1}.xvg'.format(pot.AtomTypes[0],pot.AtomTypes[1])            
            pot.Export2Gromacs(npoints=npoints, Umax=Umax,Rmaxtable=Rmaxtable, noplot=noplot, hardcopy=hardcopy, figsize=figsize, dpi=dpi, sigma=sigma, zeroforce=zeroforce, interpol=interpol,ofilename=ofilename)            
        if isinstance(pot,DF.Pot_B):
            ofilename='table_b{0:<d}.{1:<}.xvg'.format(pot.BondNumber,pot.MolTypeName)
            pot.Export2Gromacs(npoints=npoints, Umax=Umax,Rmaxtable=Rmaxtable, noplot=noplot, hardcopy=hardcopy, figsize=figsize, dpi=dpi, sigma=sigma, zeroforce=zeroforce, interpol=interpol,ofilename=ofilename)            
        if isinstance(pot,DF.Pot_A):
            ofilename='table_a{0:<d}.{1:<}.xvg'.format(pot.BondNumber,pot.MolTypeName)
            pot.Export2Gromacs(npoints=npoints, Umax=Umax,Rmaxtable=PHImaxtable, noplot=noplot, hardcopy=hardcopy, figsize=figsize, dpi=dpi, sigma=sigma, zeroforce=zeroforce, interpol=interpol,ofilename=ofilename)
                
    print '\nYou can use the following lines as a template to specify non-bonded interactions to your mdp-file'
    print '\nBut DO NOT FORGET TO ADJUST THEM MANUALLY !!!'
    print '\nenergygrps ='+' '.join(pots.Types)
    print 'energygrp_table ='+' '.join(['{0} {1}'.format(pot.AtomTypes[0], pot.AtomTypes[1]) for pot in pots if pot.Type=='NB'])

def Deviation(filename,hardcopy=False, returnarrays=False, testpoints=False):
    '''analyzes output file (or set of files) produced by MagiC and plots deviation between set of reference 
    distribution functions and resulting distribution function obtained on every iteration of inverse procedure.
    filename - name of the magic output file or list of such names (mandatory argument)
    hardcopy - if the plot should be saved to a .eps file (optional argument)
    testpoints - if the points sampled in intermediate convergence tests shall be also plotted.
    returnarrays -if true, the procedure also returns numpy arrays with iteration number and deviation values
    '''
    import re    
    import numpy as np

    TestDevListNorm=[]
    TestDevList=[]
    DevList=[]
    DevListNorm=[]
    itr=[]
    def __GetDevListFromFile__(filename,TestDevList,TestDevListNorm,DevList,DevListNorm,itr):
        ifile=open(filename,'r')
        lines=ifile.readlines()
        ifile.close()
        TestDevNorm=[]
        TestDev=[]
        
        cnt=0
        def f(x):
            return re.search('Deviation',x)
        l=filter(f,lines)        
        for line in l:            
            if line.count(':')==3: #new format of deviation output line
                if 'test' in line:
                    TestDevNorm.append(float(line.split('\n')[0].split('RDF:')[1]))
                    TestDev.append(float(line.split('\n')[0].split('S:')[1].split('RDF:')[0]))
                else:
                    TestDevListNorm.append(TestDevNorm)
                    TestDevNorm=[]
                    TestDevList.append(TestDev)
                    TestDev=[]
                    DevListNorm.append(float(line.split('\n')[0].split('RDF:')[1]))
                    DevList.append(float(line.split('\n')[0].split('S:')[1].split('RDF:')[0]))                    
                    cnt+=1
                    itr.append(filename+'.iter'+str(cnt))
            else: # old format of deviation output lines
                if 'test' in line:
                    TestDevNorm.append(float(line.split('\n')[0].split('norm:')[1]))
                    TestDev.append(float(line.split('\n')[0].split('RDF')[1].split('norm:')[0]))
                else:
                    TestDevListNorm.append(TestDevNorm)
                    TestDevNorm=[]
                    TestDevList.append(TestDev)
                    TestDev=[]
                    DevListNorm.append(float(line.split('\n')[0].split('norm:')[1]))
                    DevList.append(float(line.split('\n')[0].split('RDF')[1].split('norm:')[0]))    
                    cnt+=1
                    itr.append(filename+'.iter'+str(cnt))
        return    
    def __List2ScatterArray__(List):
        XtestList=[]
        YtestList=[]
        for i in range(0,len(List)):
            YtestList=YtestList+List[i]
            XtestList=XtestList+[i+1]*len(List[i])
        return(np.array(XtestList),np.array(YtestList))
    if isinstance(filename,type([])):
        for fil in filename:
            __GetDevListFromFile__(fil,TestDevList,TestDevListNorm,DevList,DevListNorm,itr)
    else:
        __GetDevListFromFile__(filename,TestDevList,TestDevListNorm,DevList,DevListNorm,itr)
    for i in xrange(0,len(DevList)):
        print(itr[i]+('    S: %9.5f       RDF: %9.5f' %(DevList[i],DevListNorm[i])))
    fig=plt.figure()#figsize=figsize,dpi=dpi)
    ax1=fig.add_subplot(211)
    plt.title('Deviation: $S_{ref}-S_{iter}$')
    ax2=fig.add_subplot(212)
    if testpoints:
        ax1.plot(__List2ScatterArray__(TestDevList)[0],__List2ScatterArray__(TestDevList)[1],'bo',label='Intermediate result',mfc='none')
    ax1.plot(np.array(range(1,len(DevList)+1)),np.array(DevList),'bo',label='$\sum_{r_j}[S_{ref}(r_j)-S_{(i)}(r_j)]^2$')
    ax1.set_xlabel('Iteration')
    ax1.set_ylabel('$\sum_{r_j}[S_{ref}(r_j)-S_{(i)}(r_j)]^2$')        
    if testpoints:
        ax2.plot(__List2ScatterArray__(TestDevListNorm)[0],__List2ScatterArray__(TestDevListNorm)[1],'ro',label='Intermediate result',mfc='none')
    ax2.plot(np.array(range(1,len(DevListNorm)+1)),np.array(DevListNorm),'ro',label='$\sum_{r_j}{[g_{ref}(r_j)-g_{(i)}(r_j)]^2}$')
    ax2.set_xlabel('Iteration')
    ax2.set_ylabel('$\sum_{r_j}{[g_{ref}(r_j)-g_{(i)}(r_j)]^2}$')                                    
    ax1.legend(loc=0)
    ax2.legend()
    if hardcopy:
        plt.savefig('deviation.eps')
    plt.show()
    if returnarrays:
        if testpoints:
            return __List2ScatterArray__(TestDevList)[0],__List2ScatterArray__(TestDevList)[1], __List2ScatterArray__(TestDevListNorm)[0],__List2ScatterArray__(TestDevListNorm)[1]
        else:
            return __List2ScatterArray__(DevList)[0],__List2ScatterArray__(DevList)[1], __List2ScatterArray__(DevListNorm)[0],__List2ScatterArray__(DevListNorm)[1]
def __GromacsTopology(mcmfile,topfile='topol.top'):
    ''' !!!Obsolete version!!!
    Creates a Gromacs topology file *.top from a given of mcm-file or a list of mcm-files.
    Number of molecules in the resulting system sould be stated manually once the top-file is created.
    Example 1: Single molecular type: MagicTools.GromacsTopology('DMPC.CG.mcm')
    Example 2: Multiple molecular types: MagicTools.GromacsTopology(['DMPC.CG.mcm','CHOL.CG.mcm'], topfile='DMPC-CHOL.top')
    '''
    from collections import defaultdict
    ofile=open(topfile,'w')
# writing head to a top-file
    ofile.write('; This topology file was automatically generated by MagicTools.GromacsTopology, so use it with care!\n')
    ofile.write('[ defaults ]\n')
    ofile.write('; nbfunc	comb-rule	gen-pairs	fudgeLJ	fudgeQQ\n   1		1		no		1.0	1.0 \n')
    if not (isinstance(mcmfile,type([])) or isinstance(mcmfile,type(()))):
        mcmfile=(mcmfile,)
# Writing information about atomtypes    
    ofile.write("[ atomtypes ]\n;!!!THIS ATOMTYPES DEFINITION ARE ARBITRARY AND STATED HERE TO COMPLY GROMACS REQUIREMENTS\n; THEY WILL BE OVERDEFINED BY ATOMS SECTION BELOW!!!\n;name  at.num      mass        charge   ptype       c6           c12\n")
    iBondGlobal=0
    for imcmfile in mcmfile: 
        ifile=open(imcmfile,'r')
        l=ifile.readlines()
        # cut away all lines starting with # or !
        l=map(lambda i: i.replace('\n','').strip(),l)
        l = map(lambda s: s.lstrip(), l)
        l = filter(lambda s: not s.startswith(('#','!')),l)        
        l = filter(lambda s: s!='',l)        

        NAtoms=int(l.pop(0).replace('\n',''))
        s=set(map(lambda x: x.replace('\n','').split()[7],l[0:NAtoms]))
        AtomTypes=list(s)
        AtomTypes.sort()
        for i in xrange(0,len(AtomTypes)):
            ofile.write("%4s %4i %8.4f %6.3f A  1.00  0.000 \n" %(AtomTypes[i],1,1.0, 0.0))
        ifile.close()

    for imcmfile in mcmfile: # Writing information about each molecular type
        ifile=open(imcmfile,'r')
        l=ifile.readlines()
        l=map(lambda i: i.replace('\n','').strip(),l)
        l = map(lambda s: s.strip(), l)
        l = map(lambda s: s.lstrip(), l)
        l = filter(lambda s: not s.startswith(('#','!')),l)        
        l = filter(lambda s: s!='',l)        
        lines=l
        NAtoms=int(lines.pop(0))
        AtomsName={}
        AtomsCharge={}
        AtomsCoords={}
        AtomsMass={}
        AtomsType={}
        AtomTypesCharge={} # Charge of the given atom type
        AtomsTypeName={}
        for i in xrange(0,NAtoms):
            AtomsName[i+1]=lines[i].split()[0]
            AtomsCoords[i+1]=map(float,lines[i].split()[1:4]) # in A
            AtomsMass[i+1]=float(lines[i].split()[4])
            AtomsCharge[i+1]=float(lines[i].split()[5])
            AtomsType[i+1]=int(lines[i].split()[6])
        GotAtomNames=True
        for i in xrange(1,NAtoms+1):
            AtomTypesCharge[AtomsType[i]]=AtomsCharge[i]
        if len(lines[0].split())==8:        
            GotTypeNames=True
            for i in xrange(0,NAtoms):
                AtomsTypeName[i+1]=lines[i].split()[7]
            AtomTypes={}        
            for i in xrange(1,NAtoms+1):
                AtomTypes[AtomsType[i]]=AtomsTypeName[i]
        else:
            GotTypeNames=False
        for i in xrange(0,NAtoms): # remove lines with atom details
            lines.pop(0)
    # moleculetype
        ofile.write('[ moleculetype ]\n; molname	nrexcl\n')    
        ofile.write(imcmfile.split('.mcm')[0]+' 1 \n')
        # atoms
        ofile.write('[ atoms ] \n;   nr   type  resnr residue  atom   cgnr     charge       mass\n')
        for i in xrange(1,NAtoms+1):
            ofile.write("%6i %6s %6i %6s %5s %5i %5.3f %8.4f \n" %(i,AtomsTypeName[i],1,imcmfile.split('.mcm')[0],AtomsName[i], i, AtomsCharge[i], AtomsMass[i]))
        # bonds
        ofile.write("[ bonds ]\n ; i	j	funct	table(bond type) k \n")
        NBonds=int(lines.pop(0))    
        for i in xrange(0,NBonds):
            iBondGlobal+=1
            NPairs=int(lines.pop(0)) # number of pairs in the current bond
            for j in xrange(0,NPairs):
                l=lines.pop(0)                
                ofile.write("%3i %3i %3i %3i %4.2f \n" %(int(l.split()[0]),int(l.split()[1]),8,iBondGlobal,1.0))
        # angles
        ofile.write("[ angles ]\n ; i	j   k	funct	table(bond type) k \n")
        NAngles=int(lines.pop(0))    
        pairs=[]
        for i in xrange(0,NAngles):
            NTriplets=int(lines.pop(0)) # number of pairs in the current bond
            iBondGlobal+=1
            for j in xrange(0,NTriplets):
                l=lines.pop(0)                
                pairs.append([int(l.split()[0]),int(l.split()[1])])
                pairs.append([int(l.split()[1]),int(l.split()[0])])
                ofile.write("%3i %3i %3i %3i %3i %4.2f \n" %(int(l.split()[0]),int(l.split()[2]),int(l.split()[1]),8,iBondGlobal,1.0))
        pairs.sort()
        d= defaultdict( list ) # Dictionary with exclusions
        for k,v in pairs:
            d[k].append(v)
        
        ofile.write('[ exclusions ] \n')
        for i in d:
            ofile.write(str(i)+' ')
            for j in d[i]:
                ofile.write(str(j)+' ')
            ofile.write('\n')
        ifile.close()
    ofile.write('[ system ]\n')
    ofile.write('!!! WRITE HERE THE NAME OF YOUR SYSTEM !!!\n')    
    ofile.write('[ molecules ]\n')
    for imcmfile in mcmfile:
        ofile.write(imcmfile.split('.mcm')[0]+' !!! WRITE HERE THE NUMBER OF MOLECULES !!!\n')        
    ofile.close()
    print "Topology created sucessfully. Do not forget to manually add number of molecules at the end of the file."
    
def GromacsTopology(mcmfile,topfile='topol.top'):
    ''' Creates a Gromacs topology file *.top from a given of mcm-file or a list of mcm-files.
    Number of molecules in the resulting system sould be stated manually once the top-file is created.
    Example 1: Single molecular type: MagicTools.GromacsTopology('DMPC.CG.mcm')
    Example 2: Multiple molecular types: MagicTools.GromacsTopology(['DMPC.CG.mcm','CHOL.CG.mcm'], topfile='DMPC-CHOL.top')
    '''    
    from collections import defaultdict
    import MolType
    
    ofile=open(topfile,'w')
# writing head to a top-file
    ofile.write('; This topology file was automatically generated by MagicTools.GromacsTopology, so use it with care!\n')
    ofile.write('[ defaults ]\n')
    ofile.write('; nbfunc	comb-rule	gen-pairs	fudgeLJ	fudgeQQ\n   1		1		no		1.0	1.0 \n')
    if not (isinstance(mcmfile,type([])) or isinstance(mcmfile,type(()))):
        mcmfile=(mcmfile,)
# Writing information about atomtypes    
    ofile.write("[ atomtypes ]\n;!!!THIS ATOMTYPES DEFINITION ARE ARBITRARY AND STATED HERE TO COMPLY GROMACS REQUIREMENTS\n; THEY WILL BE OVERDEFINED BY ATOMS SECTION BELOW!!!\n;name  at.num      mass        charge   ptype       c6           c12\n")
    iBondGlobal=0
    
    MType=[MolType.MolType(imcmfile) for imcmfile in mcmfile]
    
    for at in set([A.Type for MT in MType for A in MT.Atoms]):
        ofile.write("{0:4s} {1:4d} {2:8.4f} {3:6.3f} A  1.00  0.000 \n".format(at,1,1.0, 0.0))
    
    for MT in MType:
        ofile.write('[ moleculetype ]\n; molname	nrexcl\n')    
        ofile.write('{0} 1\n'.format(MT.Name))        
        # atoms
        ofile.write('[ atoms ] \n;   nr   type  resnr residue  atom   cgnr     charge       mass\n')
        for i,A in enumerate(MT.Atoms):            
            ofile.write("{0:6d} {1:6s} {2:6d} {3:6s} {4:5s} {5:5d} {6:5.3f} {7:8.4f} \n".format(i+1,A.Type,1,MT.Name,A.Name, i+1, A.Charge, A.Mass))
        # bonds
        ofile.write("[ bonds ]\n ; i	j	funct	table(bond type) k \n")
        for i,B in enumerate(MT.Bonds):
            iBondGlobal+=1
            for AG in B.AtomGroups:
                ofile.write("{0:3d} {1:3d} {2:3d} {3:3d} {4:4.2f} \n".format(AG[0],AG[1],8,iBondGlobal,1.0))
        
        # angles
        ofile.write("[ angles ]\n ; i	j   k	funct	table(bond type) k \n")
        pairs=[]
        for i, A in enumerate(MT.ABonds):
            iBondGlobal+=1
            for AG in A.AtomGroups:
                ofile.write("{0:3d} {1:3d} {2:3d} {3:3d} {4:3d} {5:4.2f} \n".format(AG[0],AG[1],AG[2],8,iBondGlobal,1.0))
                pairs.append([AG[0],AG[2]])
                pairs.append([AG[2],AG[0]])
        pairs.sort()
        d= defaultdict( list ) # Dictionary with exclusions
        for k,v in pairs:
            d[k].append(v)            
        ofile.write('[ exclusions ] \n')
        for i in d:
            ofile.write(str(i)+' ')
            for j in d[i]:
                ofile.write(str(j)+' ')
            ofile.write('\n')

    ofile.write('[ system ]\n')
    ofile.write('!!! WRITE HERE THE NAME OF YOUR SYSTEM !!!\n')    
    ofile.write('[ molecules ]\n')
    for MT in MType:
        ofile.write(MT.Name+' !!! WRITE HERE THE NUMBER OF MOLECULES !!!\n')        
    ofile.close()
    print "Topology created sucessfully. Do not forget to manually add number of molecules at the end of the file."    

def Convert2NewFile_rdf(rdffile,mcmfile):
    ''' Convert an old *.rdf file from MagiC-1.0 format to the MagiC-2.0 file format. The generated rdf-file will have same name as an old one, with additional suffix ".v2."
    rdffile - filename of the original "old" ver 1.0 file
    mcmfile - file or list of mcm-files (or mmol-files), in the SAME ORDER as they were introduced for rdf-calcualtion        
    '''

    import numpy
    import sys
    import MolType

    if not (isinstance(mcmfile,type([])) or isinstance(mcmfile,type(()))):
        mcmfile=(mcmfile,)

    MTypes=map(lambda s: MolType.MolType(s), mcmfile)    
    NAtoms=map(lambda x: x.NAtom, MTypes)

    def getMolTypeInd(ind,NAtoms):
        i=1
        for iMType in xrange(0,len(NAtoms)):
            for atom in xrange(0,NAtoms[iMType]):
                if ind==i:
                    return (atom+1, iMType+1) #return relative atom number and MolType number
                i+=1                    

    ifile=open(rdffile,'r')
    lines=ifile.readlines()
    ifile.close()
    lines = map(lambda s: s.strip(), lines)
    lines = filter(lambda s: (s[0]!='!')&(s[0]!='#'),lines)

    ofile=open(rdffile.split('.rdf')[0]+'.v2.rdf','w')      

    NTypes=int(lines[0].split()[0])
    Types=lines[1:NTypes+1]
    RMin=float(lines[0].split()[2])
    RMax=float(lines[0].split()[3])
    NA=int(lines[0].split()[1])
    print Types
    rdfs=__GetDFsFromFile_rdf(rdffile)
    rdfs_NB=filter(lambda rdf: rdf.Type=='NB', rdfs)
    rdfs_B=filter(lambda rdf: (rdf.Type=='B'), rdfs)
    rdfs_A=filter(lambda rdf: (rdf.Type=='A'), rdfs)

    olines=[]
    olines.append('&General')
    olines.append("NTypes={0:d}".format(NTypes))             
    olines.append("Types = "+', '.join(Types))
    olines.append("N_NB={0:d}".format(len(rdfs_NB)))
    olines.append("N_B={0:d}".format(len(rdfs_B)))
    olines.append("N_A={0:d}".format(len(rdfs_A)))
    olines.append("Max={0:f}".format(RMax))
    olines.append("Min={0:f}".format(RMin))
    olines.append("NPoints={0:d}".format(NA))
    olines.append('&EndGeneral')

    def write_NB(rdf):
        olines_NB=[]
        olines_NB.append('&RDF')
        olines_NB.append("Name={}".format(rdf.Name))             
        olines_NB.append("AtomTypes={}".format(', '.join(rdf.AtomTypes)))             
        olines_NB.append("Type = NB")
        olines_NB.append("Min={0:f}".format(rdf.Min))
        olines_NB.append("Max={0:f}".format(rdf.Max))
        olines_NB.append("NPoints={0:d}".format(rdf.Npoints))
        olines_NB.append('&Table')
        prnt_rdf=rdf.g[numpy.where(rdf.g[:,0]!=0)]
        for i in xrange(0,len(prnt_rdf)):
            olines_NB.append( ' '.join((str(prnt_rdf[i,0]),(str(prnt_rdf[i,1])))))
        olines_NB.append('&EndTable')
        olines_NB.append('&EndRDF')
        return olines_NB
    def write_B(rdf,bond):
        olines_B=[]
        olines_B.append('&RDF')
        olines_B.append("Name={}".format(rdf.Name))             
        olines_B.append("Type = B")
        olines_B.append("MolType = {}".format(bond.MolType.Name))
        olines_B.append("NPairs={0:d}".format(len(bond.AtomGroups)))
        olines_B.append("Pairs={}".format(', '.join(['-'.join(map(str,bond.AtomGroups[i][:])) for i in xrange(0,len(bond.AtomGroups))])))   
        olines_B.append("Bondnumber={0:d}".format(bond.ID))
        olines_B.append("Min={0:f}".format(rdf.Min))
        olines_B.append("Max={0:f}".format(rdf.Max))
        olines_B.append("NPoints={0:d}".format(rdf.Npoints))
        olines_B.append('&Table')
        prnt_rdf=rdf.g[numpy.where(rdf.g[:,0]!=0)]
        for i in xrange(0,len(prnt_rdf)):
            olines_B.append( ' '.join((str(prnt_rdf[i,0]),(str(prnt_rdf[i,1])))))
        olines_B.append('&EndTable')
        olines_B.append('&EndRDF')
        return olines_B

    def write_A(rdf,bond):
        olines_A=[]
        olines_A.append('&RDF')
        olines_A.append("Name={}".format(rdf.Name))             
        olines_A.append("Type = A")
        olines_A.append("MolType = {}".format(bond.MolType.Name))
        olines_A.append("NTriplets={0:d}".format(len(bond.AtomGroups)))
        olines_A.append("Triplets={}".format(', '.join(['-'.join(map(str,bond.AtomGroups[i][:])) for i in xrange(0,len(bond.AtomGroups))])))   
        olines_A.append("Bondnumber={0:d}".format(bond.ID))
        olines_A.append("Min={0:f}".format(rdf.Min))
        olines_A.append("Max={0:f}".format(rdf.Max))
        olines_A.append("NPoints={0:d}".format(rdf.Npoints))
        olines_A.append('&Table')
        prnt_rdf=rdf.g[numpy.where(rdf.g[:,0]!=0)]
        for i in xrange(0,len(prnt_rdf)):
            olines_A.append( ' '.join((str(prnt_rdf[i,0]),(str(prnt_rdf[i,1])))))
        olines_A.append('&EndTable')
        olines_A.append('&EndRDF')
        return olines_A

    olines=map(lambda s: s+'\n', olines)
    ofile.writelines(olines)
    for rdf in rdfs_NB:
         olines=map(lambda s: s+'\n', write_NB(rdf))
         ofile.writelines(olines)

    for rdf in rdfs_B:
    # Get Atom groups involved in the bond, with atoms indexed in relative numbers    
        AGroups=map(lambda x: [getMolTypeInd(x[0],NAtoms)[0],getMolTypeInd(x[1],NAtoms)[0]], rdf.AtomGroups)
    # Check if all the atoms belong to the same molecular type
        MTypesGroup=map(lambda x: [getMolTypeInd(x[0],NAtoms)[1],getMolTypeInd(x[1],NAtoms)[1]], rdf.AtomGroups)
        iMType=MTypesGroup[0][0]
        if MTypesGroup[0][1]!=iMType:
            print "ERROR: error in RDF for pairwise bond.",rdfs_B.index(rdf)+1," Atoms in the bond belongs to different molecular types"
            sys.exit()
        for i in MTypesGroup:
            if i!=MTypesGroup[0]:
                print "ERROR: error in RDF for pairwise bond.",rdfs_B.index(rdf)+1," Atoms in the bond belongs to different molecular types"
                sys.exit()
        MType=MTypes[iMType-1]
    #  Search for the according bond in the Molecular Type
        try:        
            i=map(lambda x: x.AtomGroups, MType.Bonds).index(AGroups)        
        except:
            print "ERROR: error in RDF for pairwise bond.",rdfs_B.index(rdf)+1," Can not find such bond in the molecular type description. Check lists of atom pairs stated for the bond in rdf and mcm files"            
            sys.exit()                    
        olines=map(lambda s: s+'\n', write_B(rdf,MType.Bonds[i]))
        ofile.writelines(olines)

    for rdf in rdfs_A:
    # Get Atom groups involved in the bond, with atoms indexed in relative numbers    
        AGroups=map(lambda x: [getMolTypeInd(x[0],NAtoms)[0],getMolTypeInd(x[1],NAtoms)[0],getMolTypeInd(x[2],NAtoms)[0]], rdf.AtomGroups)
    # Check if all the atoms belong to the same molecular type
        MTypesGroup=map(lambda x: [getMolTypeInd(x[0],NAtoms)[1],getMolTypeInd(x[1],NAtoms)[1],getMolTypeInd(x[2],NAtoms)[1]], rdf.AtomGroups)
        iMType=MTypesGroup[0][0]
        if (MTypesGroup[0][1]!=iMType) | (MTypesGroup[0][2]!=iMType):
            print "ERROR: error in RDF for angle bond.",rdfs_A.index(rdf)+1," Atoms in the bond belongs to different molecular types"
            sys.exit()
        for i in MTypesGroup:
            if i!=MTypesGroup[0]:
                print "ERROR: error in RDF for angle bond.",rdfs_A.index(rdf)+1," Atoms in the bond belongs to different molecular types"
                sys.exit()
        MType=MTypes[iMType-1]
    #  Search for the according bond in the Molecular Type
        try:        
            i=map(lambda x: x.AtomGroups, MType.ABonds).index(AGroups)        
        except:
            
            print "ERROR: error in RDF for angle bond.",rdfs_A.index(rdf)+1," Can not find such bond in the molecular type description. Check lists of atom triplets stated for the bond in rdf and mcm files"
            sys.exit()                    
        olines=map(lambda s: s+'\n', write_A(rdf,MType.ABonds[i]))
        ofile.writelines(olines)
    #    
    print 'Converted succesfully: {0} --> {1}'.format(rdffile, ofile.name)
    ofile.close()
def Convert2NewFile_pot(potfile,mcmfile):
    ''' Converts an .pot-file produced by MagiC 1.0 to the .v2.pot file accepted by MagiC 2.0
    potfile - filename of the original "old" ver. 1.0 file
    mcmfile - file or list of mcm-files (or mmol-files), in the SAME ORDER as they were introduced in input file for MagiC, where potential calcualtion was performed
    The pot-file will have same name as an old one, with additional suffix .v2.
    '''
    import sys
    import MolType
    
    if not (isinstance(mcmfile,type([])) or isinstance(mcmfile,type(()))):
        mcmfile=[mcmfile]
    
    MTypes=map(lambda s: MolType.MolType(s), mcmfile)    
    NAtoms=map(lambda x: x.NAtom, MTypes)
    latoms=[]
    for iMType in MTypes:
        latoms=latoms+iMType.Atoms
    latoms=set(latoms)

    def getMolTypeInd(ind,NAtoms):
        i=1
        for iMType in xrange(0,len(NAtoms)):
            for atom in xrange(0,NAtoms[iMType]):
                if ind==i:
                    return (atom+1, iMType+1) #return relative atom number and MolType number
                i+=1                            
    ifile=open(potfile,'r')
    ofile=open(potfile.split('.pot')[0]+'.v2.pot','w')  
    lines=ifile.readlines()
    ifile.close()
    lines = map(lambda s: s.strip(), lines)
    lines = filter(lambda s: (s[0]!='!')&(s[0]!='#'),lines)
    
    NTypes=int(lines[0].split()[0])
    RMin=float(lines[0].split()[5])
    RMax=float(lines[0].split()[6])
    NA=int(lines[0].split()[1])

    pots=__GetPotsFromFile_pot(potfile,mcmfile=mcmfile,Ucut=1000000)
    pots_NB=filter(lambda pot: pot.Type=='NB', pots)
    pots_B=filter(lambda pot: (pot.Type=='B'), pots)
    pots_A=filter(lambda pot: (pot.Type=='A'), pots)
    
    olines=[]
    olines.append('&General')
    olines.append("NTypes={0:d}".format(NTypes))             
    olines.append("N_NB={0:d}".format(len(pots_NB)))
    olines.append("N_B={0:d}".format(len(pots_B)))
    olines.append("N_A={0:d}".format(len(pots_A)))
    olines.append("Max={0:f}".format(RMax))
    olines.append("Min={0:f}".format(RMin))
    olines.append("NPoints={0:d}".format(NA))
    olines.append('&EndGeneral')
    
    def write_NB(pot):
        olines_NB=[]
        olines_NB.append('&Potential')
        olines_NB.append("Name={}".format(pot.Name))                 
        olines_NB.append("AtomTypes={}".format(', '.join(pot.AtomTypes)))             
        olines_NB.append("Type = NB")
        olines_NB.append("Min={0:f}".format(pot.Min))
        olines_NB.append("Max={0:f}".format(pot.Max))
        olines_NB.append("NPoints={0:d}".format(pot.Npoints))
        olines_NB.append('&Table')
        prnt_pot=pot.g#[numpy.where(pot.g[:,0]!=0)]
        for i in xrange(0,len(prnt_pot)):
            olines_NB.append( ' '.join((str(prnt_pot[i,0]),(str(prnt_pot[i,1])))))
        olines_NB.append('&EndTable')
        olines_NB.append('&EndPotential')
        return olines_NB
    def write_B(pot,bond):
        olines_B=[]
        olines_B.append('&Potential')
        olines_B.append("Name={}".format(pot.Name))             
        olines_B.append("Type = B")
        olines_B.append("MolType = {}".format(bond.MolType.Name))
        olines_B.append("NPairs={0:d}".format(len(bond.AtomGroups)))
        olines_B.append("Pairs={}".format(', '.join(['-'.join(map(str,bond.AtomGroups[i][:])) for i in xrange(0,len(bond.AtomGroups))])))   
        olines_B.append("Bondnumber={0:d}".format(bond.ID))
        olines_B.append("Min={0:f}".format(pot.Min))
        olines_B.append("Max={0:f}".format(pot.Max))
        olines_B.append("NPoints={0:d}".format(pot.Npoints))
        olines_B.append('&Table')
        prnt_pot=pot.g#[numpy.where(pot.g[:,0]!=0)]
        for i in xrange(0,len(prnt_pot)):
            olines_B.append( ' '.join((str(prnt_pot[i,0]),(str(prnt_pot[i,1])))))
        olines_B.append('&EndTable')
        olines_B.append('&EndPotential')
        return olines_B
    
    def write_A(pot,bond):
        olines_A=[]
        olines_A.append('&Potential')
        olines_A.append("Name={}".format(pot.Name))             
        olines_A.append("Type = A")
        olines_A.append("MolType = {}".format(bond.MolType.Name))
        olines_A.append("NTriplets={0:d}".format(len(bond.AtomGroups)))
        olines_A.append("Triplets={}".format(', '.join(['-'.join(map(str,bond.AtomGroups[i][:])) for i in xrange(0,len(bond.AtomGroups))])))   
        olines_A.append("Bondnumber={0:d}".format(bond.ID))
        olines_A.append("Min={0:f}".format(pot.Min))
        olines_A.append("Max={0:f}".format(pot.Max))
        olines_A.append("NPoints={0:d}".format(pot.Npoints))
        olines_A.append('&Table')
        prnt_pot=pot.g#[numpy.where(pot.g[:,0]!=0)]
        for i in xrange(0,len(prnt_pot)):
            olines_A.append( ' '.join((str(prnt_pot[i,0]),(str(prnt_pot[i,1])))))
        olines_A.append('&EndTable')
        olines_A.append('&EndPotential')
        return olines_A
    
    olines=map(lambda s: s+'\n', olines)
    ofile.writelines(olines)
    for pot in pots_NB:
         olines=map(lambda s: s+'\n', write_NB(pot))
         ofile.writelines(olines)
    
    for pot in pots_B:
    # Get Atom groups involved in the bond, with atoms indexed in relative numbers    
        AGroups=map(lambda x: [getMolTypeInd(x[0],NAtoms)[0],getMolTypeInd(x[1],NAtoms)[0]], pot.AtomGroups)
    # Check if all the atoms belong to the same molecular type
        MTypesGroup=map(lambda x: [getMolTypeInd(x[0],NAtoms)[1],getMolTypeInd(x[1],NAtoms)[1]], pot.AtomGroups)
        iMType=MTypesGroup[0][0]
        if MTypesGroup[0][1]!=iMType:
            print "ERROR: error in potential for pairwise bond ",pots_B.index(pot)+1,". Atoms in the bond belongs to different molecular types"
            sys.exit()
        for i in MTypesGroup:
            if i!=MTypesGroup[0]:
                print "ERROR: error in potential for pairwise bond ",pots_B.index(pot)+1,". Atoms in the bond belongs to different molecular types"
                sys.exit()
        MType=MTypes[iMType-1]
    #  Search for the according bond in the Molecular Type
        i=-1
        grps=[[ag for ag in bnd.AtomGroups] for bnd in MType.Bonds]
        inv_grps=[ [[g[1],g[0]] for g in grp] for grp in grps]
        for ii in range(len(grps)):      
            if all(A in grps[ii] for A in AGroups):
                i=ii
            if all(A in inv_grps[ii] for A in AGroups):
                i=ii                
        if i==-1:
            print "ERROR: error in potential for pairwise bond ",pots_B.index(pot)+1,". Can not find such bond in the molecular type description. Check lists of atom pairs stated for the bond in pot and mcm files"
            print 'Bonded atom pairs:', AGroups
            print 'Check the order of atoms in the pair.' 
            0/0
            sys.exit()                    
             
        olines=map(lambda s: s+'\n', write_B(pot,MType.Bonds[i]))
        ofile.writelines(olines)
    
    for pot in pots_A:
    # Get Atom groups involved in the bond, with atoms indexed in relative numbers    
        AGroups=map(lambda x: [getMolTypeInd(x[0],NAtoms)[0],getMolTypeInd(x[1],NAtoms)[0],getMolTypeInd(x[2],NAtoms)[0]], pot.AtomGroups)
    # Check if all the atoms belong to the same molecular type
        MTypesGroup=map(lambda x: [getMolTypeInd(x[0],NAtoms)[1],getMolTypeInd(x[1],NAtoms)[1],getMolTypeInd(x[2],NAtoms)[1]], pot.AtomGroups)
        iMType=MTypesGroup[0][0]
        if (MTypesGroup[0][1]!=iMType) | (MTypesGroup[0][2]!=iMType):
            print "ERROR: error in potential for angle bond.",pots_A.index(pot)+1," Atoms in the bond belongs to different molecular types"
            sys.exit()
        for i in MTypesGroup:
            if i!=MTypesGroup[0]:
                print "ERROR: error in potential for angle bond.",pots_A.index(pot)+1," Atoms in the bond belongs to different molecular types"
                sys.exit()
        MType=MTypes[iMType-1]
    #  Search for the according bond in the Molecular Type
        i=-1
        grps=[[ag for ag in bnd.AtomGroups] for bnd in MType.ABonds]
        inv_grps=[[[g[2],g[1],g[0]] for g in grp] for grp in grps]
        for ii in range(len(grps)):
            if all(A in grps[ii] for A in AGroups):
                i=ii
            if all(A in inv_grps[ii] for A in AGroups):
                i=ii 
        if i==-1:
            print "ERROR: error in potential for angle bond.",pots_A.index(pot)+1," Can not find such bond in the molecular type description. Check lists of atom triplets stated for the bond in pot and mcm files"
            print 'Bonded atom triplets:', AGroups
            print 'Check the order of atoms in the triplet.' 
            sys.exit()                    

        olines=map(lambda s: s+'\n', write_A(pot,MType.ABonds[i]))
        ofile.writelines(olines)
    #    
    print 'Converted succesfully: {0} --> {1}'.format(potfile, ofile.name)
    ofile.close()

def SplitDFset(ifile,ofile, Split=True):
    ''' 
    Splits the set of RDFs or potentials given in the file ifile into a main header file and an additional set of included file. Optional agrument Split cna specify a logical list, stating which DFs within the file shall be moved to inclded file, while other will remain in the main header file.
    '''
    import DFset
    DFset.DFset(ifile).Write(ofile,Split=Split)

def ReadMagiC(ifile,iters=None,DFType='RDF',PairNamesList=None,mcmfile=None):
    '''
    Parse the file <ifile> produced by MagiC software and return a list of DFset (sets of distribution functions: RDF/potential etc.): DFs[iteration][pair]
    DFType - defines function of interest: 'RDF': radial (and angle) distribution functions obtained in inverse MCcalculation; 'RDFref': reference RDFs(ADFs); 'Pot': Potential used in current iteration; 'PotNew': potential generated in current iteration; 'PotCorr': Potential correction applied at the iteration.
    iters - tuple of iterations to extract from the file. If nothing mentioned all iterations will be extracted.  Example: iters=(1,2,3) or iters=(1)
    basename - prefix given to all the DFs readed from the file    
    PairNamesList - List of pairs of atomic names to search in output file. It consists of three sublists: first of them refer to non-bonded pair interactions, the second refers to bonded pair interactions, and the third refers to bending angle (1-3) bond interactions
    E.g. PairNamesList=[['N-N','N-P','N-CH','N-CO','P-P','P-CH','P-CO','CH-CH','CH-CO','CO-CO'], ['N-P','P-C1','C2-C3','C1-C2','C1-C5'], ['N-C1-P','P-C2-C1','C1-C3-C2','C2-C4-C3']]    
    '''
    import re
    import pdb
    import MolType
    import DF
    import numpy as np
    import DFset
    import sys
    # Lines below defining set of pairs/triplets of beads which are used in magic output file to mark according potential/distribution function.
    # they are defined for every molecular type. Fist sublist keeps intermolecular interactions, second - pairwise bonds, third - angle bending bonds.
    # if a molecular type name is not provided in the subroutine's input, the PairNamesList is generated automatically, but program assumes, that combinations of bead names are unique.
    if mcmfile==None:
        pass
        # Detect molecular type names
        rdffile=open(ifile,'r')
        lines=rdffile.readlines()
        rdffile.close()
        mlines=[l for l in lines for m in [re.search('MOLECULE',l)] if m] # lines with MT names
        mnames=[l.split('MOLECULE')[1].replace(' ','').split(',')[0]+'.mcm' for l in mlines]            
        if mnames==[]: # Have not got anything from the file. Try once more
            linentypes=[l for l in lines for m in [re.search('Number of species:',l)] if m]
            if linentypes!=[]:
                ntypes=int(linentypes[0].split(':')[1].strip())
                mlines=[lines[i+1:i+1+ntypes] for i in xrange(0,len(lines)) 
                    for m in [re.search('Number of molecules of each species:',lines[i])] if m]
                if mlines!=[]:
                    mnames=[l.strip().split()[2]+'.mcm' for l in mlines[0]]                    
                else:
                    print "mlines=[]"
            else:
                print "linentypes=[]"
        if mnames==[]:            
            print 'No Molecular Types detected! Please specify the mcmfiles manually!'
            sys.exit()
        print 'Detected Molecular Types:',mnames        
        MolTypes=[MolType.MolType(imcmfile) for imcmfile in set(mnames)]        

    else:
        if not isinstance(mcmfile,type([])): # single file-name provided - convert to a list
            mcmfile=[mcmfile]
        print "Reading mcmfiles:", mcmfile
        
        MolTypes=[MolType.MolType(imcmfile) for imcmfile in mcmfile]
                    
#    if mtype=='dmpc':
#        PairNamesList=[['N-N','N-P','N-CH','N-CO','P-P','P-CH','P-CO','CH-CH','CH-CO','CO-CO'],['N-P','P-C1','C2-C3','C1-C2','C1-C5'],['N-C1-P','P-C2-C1','C1-C3-C2','C2-C4-C3']]    
#    if mtype=='hxd':
#        PairNamesList=[['CH-CH'],['C1-C2'],[]]    
#    if mtype=='hxd_A':
#        PairNamesList=[['CH-CH'],['C1-C2'],['C1-C3-C2']]    
#    if mtype=='dmpc_12':
#        PairNamesList=[['N-N','N-P','N-CH','N-CO','P-P','P-CH','P-CO','CH-CH','CH-CO','CO-CO'],['N-P','P-C1','C2-C3','C1-C2','C1-C5'],[]]    
#    if mtype=='dmpc_A6':
#        PairNamesList=[['N-N','N-P','N-CH','N-CO','P-P','P-CH','P-CO','CH-CH','CH-CO','CO-CO'],['N-P','P-C1','C2-C3','C1-C2','C1-C5'],['N-C1-P','P-C2-C1','C1-C3-C2','C2-C4-C3','C2-C5-C1','C6-C1-C5']]    
#    if mtype=='dmpc_A5':
#        PairNamesList=[['N-N','N-P','N-CH','N-CO','P-P','P-CH','P-CO','CH-CH','CH-CO','CO-CO'],['N-P','P-C1','C2-C3','C1-C2','C1-C5'],['N-C1-P','P-C2-C1','C1-C3-C2','C2-C4-C3','C2-C5-C1']]    

    if PairNamesList==None:
        print 'Automatically defining list of interactions from input file. If this does not work well for your input, mnually provide PairNamesList to the procedure.\n'
        rdffile=open(ifile,'r')
        lines=rdffile.readlines()
        l=[]
        for line in lines:
            if re.search('This pair|This triplet',line):
                l.append(line.split('\n')[0].strip())
        s=set(l) # leave only one exemplar of every pair
        ll=[]
        for line in l: #order pairs as in the file
            if line in s:
                ll.append(line)
                s.remove(line)
        PairNamesList=[[],[],[]]

        def unique(seq):
            seen = set()
            seen_add = seen.add
            return [ x for x in seq if not (x in seen or seen_add(x))]        
        
        for line in ll:
            if 'intermolecular' in line:
                PairNamesList[0].append(line.split('This pair:')[1].split('intermolecular')[0].strip().replace(' ',''))                
            if re.search('This pair.*intramolecular',line):
                PairNamesList[1].append(line.split('This pair:')[1].split('intramolecular')[0].strip().replace(' ',''))
            if re.search('This triplet:',line):
                PairNamesList[2].append(line.replace('intramolecular','').split('triplet:')[1].strip().replace(' ',''))
        PairNamesList[0]=unique(PairNamesList[0])
        PairNamesList[1]=unique(PairNamesList[1])        
        PairNamesList[2]=unique(PairNamesList[2])
        print len(PairNamesList[0]),' intermolecular pairs detected ',PairNamesList[0]
        print len(PairNamesList[1]),' pairwise bonds detected ',PairNamesList[1]
        print len(PairNamesList[2]),' angles detected ',PairNamesList[2]
        
#    if basename!=None:
#        basename+=' '
        
    if DFType.upper() not in ('RDF','RDFREF','POT','NEWPOT','POTCORR'):
        print 'ERROR: Unknown type of distribution function (DF)!'
        return
    RDFsList=[]    

    def _GrepText_(DFtype='inter',header='RDF ',column=2, s=None, cls=None, Type=None, Kind=None):
                import sys
                '''
                Parsing set of lines <lines> for  regular expression <s> and returnd object of a proper class (it is always subclass of DF)
                '''
                print 'Pair '+ str(pair)
                RDF_r_List=[]
                RDF_gr_List=[]
                name=Kind+' '+Type+' '+pair+' i'+str(irep)
                # search for resulting output (Final output, not a check one)
                try:
                    for l in lines:
                      if re.search(s,l):
                        RDF_r_List.append(float(l.split()[0]))
                        RDF_gr_List.append(float(l.split()[int(column)-1]))
                except: 
                    print '\nError while parsing the file.'
                    print 'Line: {}'.format(l)
                    print 'Parsing RegExpression: {}\n'.format(s)
                    sys.exit()
                if len(RDF_r_List)==0:
                    print '\nError while parsing the file. Some tables were not found.'
                    print 'Check if iterations are stated in accordance with the file: iters=(...)'
                    print 'Parsing RegExpression: {}\n'.format(s)                
                    sys.exit()

                RDF=np.array([[RDF_r_List[i],RDF_gr_List[i]] for i in xrange(0,len(RDF_r_List))])
                RMin=RDF[:,0].min()
                RMax=RDF[:,0].max()
                Npoints=len(RDF[:,0])
                resol=(RMax-RMin)/(Npoints-1)
                RMin=RMin-resol/2
                RMax=RMax+resol/2
                for i in xrange(0,Npoints-1):
                    if (abs(RDF[i+1,0]-RDF[i,0]-resol)>=0.0001):
                        print 'Error in the table, points are not equidistant',RMax,RMin,Npoints,resol,RDF[0,i+1],RDF[0,i]
                        print 'Check the data-table, some points might be missing'
                        return                
                trdf=cls(Type=Type,Name=name,Min=RMin,Max=RMax,Npoints=Npoints,g=RDF,Kind=Kind)#,resol=resol)
                return trdf

    def AddProp2Bond(MolTypes,atoms,RDFs):
                for MT in MolTypes:
                    MTatoms=[a.Name for a in MT.Atoms]
                    if all(x in MTatoms for x in atoms):
                        print 'The bond belongs to the molecular type: {0}'.format(MT.Name)
                        IDatoms=[A.ID for a in atoms for A in MT.Atoms if (a==A.Name)]    
                        if len(atoms)==2:
                            bonds=[Bond for Bond in MT.Bonds if (IDatoms in Bond.AtomGroups)]
                        if len(atoms)==3:
                            bonds=[Bond for Bond in MT.ABonds if (IDatoms in Bond.AtomGroups)]
                        if len(bonds)>1: 
                            print 'Error: this pair of atoms is involved in more than one bond! Check your mcm-files'
                            sys.exit()
                        RDFs[-1].BondNumber=bonds[0].ID
                        RDFs[-1].MolTypeName=MT.Name
                        RDFs[-1].AtomGroups=bonds[0].AtomGroups

    
    rdffile=open(ifile,'r')
    lines=rdffile.readlines()
    # Get total number of iterations performed:
    if (iters==None):
        TotalNumIter=0
        for l in lines:
            if re.search('Final Corrected effective potential',l):
                TotalNumIter+=1
        print 'Total number of iterations:', TotalNumIter
        iters=range(1,TotalNumIter+1)
    else:
        if isinstance(iters,int): # eg. single iteration
            iters=[iters]
        TotalNumIter=len(iters)
    # Get number of checks for every iteration
    TotalNumCheck=0
    for l in lines:
        if re.search('Global Potential correction convergence check',l):
            TotalNumCheck+=1
    print 'Total number of intermediate checks per iteration:', TotalNumCheck/TotalNumIter
    
    for irep in iters: #loop over all iterations
        print 'Iteration %i' %(irep)
        RDFs=[]                            
        Type='NB'
        for pair in PairNamesList[0]:                            
            if (DFType.upper()=='RDFREF')and(iters.index(irep)==0):
                s='pot: *'+pair.split('-')[0].strip()+' *'+pair.split('-')[1].strip()+' *Final *'+str(irep)+' '
                RDFs.append(_GrepText_(DFtype='inter',header='RDFref ',column=3, s=s, Type=Type, Kind='RDFRef', cls=DF.RDFref_NB))
            else:
            #Loop over intermolecular pairs
                s='pot: *'+pair.split('-')[0].strip()+' *'+pair.split('-')[1].strip()+' *Final *'+str(irep)+' '
                if DFType.upper()=='RDF':
                    RDFs.append(_GrepText_(DFtype='inter',header='RDF ',column=2,s=s, Type=Type, Kind='RDF', cls=DF.RDF_NB))
                if DFType.upper()=='POT':
                    RDFs.append(_GrepText_(DFtype='inter',header='Pot ',column=5,s=s, Type=Type, Kind='POT', cls=DF.Pot_NB))
                if DFType.upper()=='NEWPOT':
                    RDFs.append(_GrepText_(DFtype='inter',header='Pot ',column=4,s=s, Type=Type, Kind='POT', cls=DF.Pot_NB))
                if DFType.upper()=='POTCORR':
                    RDFs.append(_GrepText_(DFtype='inter',header='PotCorr ',column=6,s=s, Type=Type, Kind='POTCORR', cls=DF.Pot_NB))
            RDFs[-1].AtomTypes=pair.split('-')

        Type='B'
        for pair in PairNamesList[1]:
            if (DFType.upper()=='RDFREF')and(iters.index(irep)==0):
                s='pot: *'+pair.split('-')[0].strip()+' *'+pair.split('-')[1].strip()+' *'+'-*[0-9] *Final *'+str(irep)+' *$'
                RDFs.append(_GrepText_(DFtype='intra',header='RDFref ',column=3, s=s, Type=Type, Kind='RDFRef', cls=DF.RDFref_B))
            else:
            #Loop over intramolecular pairs with pairwise bonds
                s='pot: *'+pair.split('-')[0].strip()+' *'+pair.split('-')[1].strip()+' *'+'-*[0-9] *Final *'+str(irep)+' *$'
                if DFType.upper()=='RDF':
                    RDFs.append(_GrepText_(DFtype='intra',header='RDF ',s=s,Type=Type, Kind='RDF', cls=DF.RDF_B))
                if DFType.upper()=='POT':
                    RDFs.append(_GrepText_(DFtype='intra',header='Pot ',column=5,s=s,Type=Type, Kind='POT', cls=DF.Pot_B))
                if DFType.upper()=='NEWPOT':
                    RDFs.append(_GrepText_(DFtype='intra',header='Pot ',column=4,s=s, Type=Type, Kind='POT', cls=DF.Pot_B))
                if DFType.upper()=='POTCORR':
                    RDFs.append(_GrepText_(DFtype='intra',header='PotCorr ',column=6,s=s,Type=Type, Kind='POTCORR', cls=DF.Pot_B))
#           print 'Addprop2bond before', MolTypes
            AddProp2Bond(MolTypes,pair.split('-'),RDFs)
        
        Type='A'                    
        for pair in PairNamesList[2]:
            if (DFType.upper()=='RDFREF')and(iters.index(irep)==0):
                s='pot: *'+pair.split('-')[0].strip()+' *'+pair.split('-')[1].strip()+' *'+pair.split('-')[2].strip()+' *'+'-*[0-9] *Final *'+str(irep)+' *$'
                RDFs.append(_GrepText_(DFtype='intra',header='ADFref ',column=3,s=s, Type=Type, Kind='RDFRef', cls=DF.RDFref_A))
            else:
                #Loop over intramolecular triplets with 1-3 bonds
                s='pot: *'+pair.split('-')[0].strip()+' *'+pair.split('-')[1].strip()+' *'+pair.split('-')[2].strip()+' *'+'-*[0-9] *Final *'+str(irep)+' *$'
                if DFType.upper()=='RDF':
                    RDFs.append(_GrepText_(DFtype='intra',header='ADF ',column=2, s=s,Type=Type, Kind='RDF', cls=DF.RDF_A))
                if DFType.upper()=='POT':
                    RDFs.append(_GrepText_(DFtype='intra',header='Pot1-3 ',column=5,s=s, Type=Type, Kind='POT', cls=DF.Pot_A))
                if DFType.upper()=='NEWPOT':
                    RDFs.append(_GrepText_(DFtype='intra',header='Pot1-3 ',column=4,s=s, Type=Type, Kind='POT', cls=DF.Pot_A))                    
                if DFType.upper()=='POTCORR':
                    RDFs.append(_GrepText_(DFtype='intra',header='PotCorr1-3 ',column=6,s=s, Type=Type, Kind='POTCORR', cls=DF.Pot_A))
            AddProp2Bond(MolTypes,pair.split('-'),RDFs)
        RDFsList.append(RDFs)

        if (DFType.upper()=='RDFREF'):
            break
        
    listDFset=[]
    for iRDFs in RDFsList: # iRDFs - single list of RDFs
        Types=list(set([at for RDF in iRDFs if RDF.Type=='NB' for at in RDF.AtomTypes]))
        listDFset.append(
        DFset.DFset(Kind=iRDFs[0].Kind, Name=ifile+'.i'+str(irep), Types=None, Min=iRDFs[0].Min, Max=iRDFs[0].Max, Npoints=iRDFs[0].Npoints, DFs=iRDFs))

    if len(listDFset)==1:
        return listDFset[0]
    else:
        return listDFset

def ReadRDF(ifile):
    '''
    Read (import) RDFs from a .rdf file (in MagiC 2.0 format) into a DFset class variable. 
    ifile - name of the file 
    '''
    return DFset.DFset(File=ifile)

def ReadPot(ifile, Ucut=10000):
    '''
    Read (import) potentials from a .pot file (in MagiC 2.0 format) into a DFset variable. 
    ifile - name of the file 
    Ucut - height of the hard repulsive core at r=0 in kJ/mol
    '''

    return DFset.DFset(ifile,Ucut=Ucut)    
    
#def GetDFsFromFile_rdf(rdffilename,iters=(),DFType='RDF',mtype='',basename='',mcmfile=None):
#    '''
#    Analyse rdf-file <rdffilename> produced by rdf-2.0 or rdf-1.3 tool and return a list of DFs.
#    basename - prefix given to all the DFs readed from the file
#    '''
#    print 'Function GetDFsFromFile_rdf is obsolete in MagiC 2.0. Please consider using the function ReadRDF instead.'
#    print 'If you still want to proceed, use __GetDFsFromFile_rdf '    
#    return

def __GetDFsFromFile_rdf(rdffilename,basename='',mcmfile=[]):
    '''
    Analyse rdf-file <rdffilename> produced by rdf-2.0 or rdf-1.3 tool and return a list of DFs.
    basename - prefix given to all the DFs readed from the file
    '''    
    import DFset
    
    if basename=='':
        basename=rdffilename
    RDFsList=[]
    rdffile=open(rdffilename,'r')
    l=rdffile.readlines()
    lines=[]
    for i in l:
        if not i.startswith('#'):
            lines.append(i)
    line=lines.pop(0)
    NAtomTypes=int(line.split()[0])
    NPointsNB=int(line.split()[1])
    RMinNB=float(line.split()[2])
    RMaxNB=float(line.split()[3])
    if NAtomTypes>0:
        resol=(RMaxNB-RMinNB)/NPointsNB
    AtomTypes=[]
    RDFs=np.zeros((NAtomTypes,NAtomTypes,NPointsNB,2))
    for i in xrange(0,NAtomTypes):
        line=lines.pop(0).split('\n')[0]
        AtomTypes.append(line.strip())
    while (lines!=[]):
        line=lines.pop(0).split('\n')[0]
        if (line.strip().upper()=='INTRA') or (line.strip().upper()=='ANGLE') or (line.strip().upper()=='END'):
             break
        r=float(line.split()[0])
        g_r=float(line.split()[1])
        i=int(line.split()[2])-1
        j=int(line.split()[3])-1
        ind=int((r-RMinNB)/resol)
        RDFs[i,j,ind,0]=r
        RDFs[i,j,ind,1]=g_r     
    for i in xrange(0,NAtomTypes):
        for j in xrange(i,NAtomTypes):
            trdf=DF.DF_NB(Type='NB',Kind='RDF',Name=basename+' RDF-NB '+AtomTypes[i]+'-'+AtomTypes[j],
                           Min=RMinNB,
                           Max=RMaxNB,
                           Npoints=NPointsNB,
                           g=RDFs[i,j,:,:], AtomTypes=[AtomTypes[i],AtomTypes[j]])
            RDFsList.append(trdf)
    if (line.strip().upper()=='INTRA'):
        while (lines!=[]):
            line=lines.pop(0).split('\n')[0]
            if (line.strip().upper()=='ANGLE') or (line.strip().upper()=='END'):
                break
            npairs=int(line.split()[0])
            NPoints=int(line.split()[1])
            RMin=float(line.split()[2])
            RMax=float(line.split()[3])
            resol=(RMax-RMin)/NPoints            
            RDF=np.zeros((NPoints,2))
            name=''
            Pairs=[]
            for i in xrange(0,npairs):
                line=lines.pop(0).split('\n')[0]
                Pairs.append([int(line.split()[0]),int(line.split()[1])])
            for i in xrange(0,NPoints):
                line=lines.pop(0).split('\n')[0]
                r=float(line.split()[0])
                g_r=float(line.split()[1])
                if name=='':
                    for k in xrange(2,len(line.replace('rdf_int:','').split())):
                        name=name+line.replace('rdf_int:','').split()[k].strip()
                ind=int((r-RMin)/resol)
                RDF[ind,0]=r
                RDF[ind,1]=g_r        
            trdf=DF.DF_B(Type='B',Kind='RDF',Name=basename+' RDF-B '+name,
                           Min=RMin,
                           Max=RMax,
                           Npoints=NPoints,
                           g=RDF, AtomGroups=Pairs)
            RDFsList.append(trdf)
            
    if (line.strip().upper()=='ANGLE'):
        while (lines!=[]):
            line=lines.pop(0).split('\n')[0]
            if (line.strip().upper()=='ANGLE') or (line.strip().upper()=='END'):
                break
            npairs=int(line.split()[0])
            NPoints=int(line.split()[1])
            RMin=float(line.split()[2])
            RMax=float(line.split()[3])
            resol=(RMax-RMin)/NPoints
            RDF=np.zeros((NPoints,2))
            name=''
            Triplets=[]
            for i in xrange(0,npairs):
                line=lines.pop(0).split('\n')[0]
                Triplets.append([int(line.split()[i]) for i in [0,2,1]])
            for i in xrange(0,NPoints):
                line=lines.pop(0).split('\n')[0]
                r=float(line.split()[0])
                g_r=float(line.split()[1])
                if name=='':
                    for k in xrange(2,len(line.split())):
                        name=name+line.split()[k].strip()
                ind=int((r-RMin)/resol)
                RDF[ind,0]=r
                RDF[ind,1]=g_r
            trdf=DF.DF_A(Type='A',Kind='RDF',Name=basename+' RDF-A '+name,
                           Min=RMin,
                           Max=RMax,
                           Npoints=NPoints,
                           g=RDF, AtomGroups=Triplets)
            RDFsList.append(trdf)
    return RDFsList
    

#def GetDFsFromFile_magic(rdffilename,iters=(),DF='RDF',mtype='',basename='',mcmfile=None):
#    print 'Function GetDFsFromFile_magic is obsolete in MagiC 2.0. Please consider using the function ReadMagic instead.'
#    print 'If you still want to proceed, use __GetDFsFromFile_magic '    
#    return
def __GetDFsFromFile_magic(rdffilename,iters=(),DF='RDF',mtype='',basename='',mcmfile=None):
    '''
    Analyse file <rdffilename> produced by MAGIC software and return a list of lists of DFs. DFs[iteration][pair]
    iters() - tuple of iterations to extract from the file. If nothing mentioned all iterations will be extracted.  Example: iters=(1,2,3) or iters=(1,) Mind the comma after single iteration number!
    basename - prefix given to all the DFs readed from the file    
    DF - defines function of interest: RDF-radial (and angle) distribution functions obtained in inverse MCcalculation ,RDFref-reference RDFs(ADFs), Pot - Potential used in a current iteration, PotNew - potential generated in current iteration.
    mtype - type of the system. It defines list of pairs of atomic names to search in output file. 
        the list consists of three sublists: first of them refer to non-bonded pair interactions, the second refers to bonded pair interactions, and the third refers to bending angle (1-3) bond interactions
        if mtype=='dmpc':
            PairNamesList=[['N-N','N-P','N-CH','N-CO','P-P','P-CH','P-CO','CH-CH','CH-CO','CO-CO'],['N-P','P-C1','C2-C3','C1-C2','C1-C5'],['N-C1-P','P-C2-C1','C1-C3-C2','C2-C4-C3']]    
        if mtype=='dmpc_12': No 1-3 interaction
            PairNamesList=[['N-N','N-P','N-CH','N-CO','P-P','P-CH','P-CO','CH-CH','CH-CO','CO-CO'],['N-P','P-C1','C2-C3','C1-C2','C1-C5'],[]]    
    '''
    import re
    import pdb    
    print "!!!\nWARNING! This function is obsolete! Use function ReadMagiC instead!\n!!!"
    # Lines below defining set of pairs/triplets of beads which are used in magic output file to mark according potential/distribution function.
    # they are defined for every molecular type. Fist sublist keeps intermolecular interactions, second - pairwise bonds, third - angle bending bonds.
    # if a molecular type name is not provided in the subroutine's input, the PairNamesList is generated automatically, but program assumes, that combinations of bead names are unique.
    if mtype=='dmpc':
        PairNamesList=[['N-N','N-P','N-CH','N-CO','P-P','P-CH','P-CO','CH-CH','CH-CO','CO-CO'],['N-P','P-C1','C2-C3','C1-C2','C1-C5'],['N-C1-P','P-C2-C1','C1-C3-C2','C2-C4-C3']]    
    if mtype=='hxd':
        PairNamesList=[['CH-CH'],['C1-C2'],[]]    
    if mtype=='hxd_A':
        PairNamesList=[['CH-CH'],['C1-C2'],['C1-C3-C2']]    
    if mtype=='dmpc_12':
        PairNamesList=[['N-N','N-P','N-CH','N-CO','P-P','P-CH','P-CO','CH-CH','CH-CO','CO-CO'],['N-P','P-C1','C2-C3','C1-C2','C1-C5'],[]]    
    if mtype=='dmpc_A6':
        PairNamesList=[['N-N','N-P','N-CH','N-CO','P-P','P-CH','P-CO','CH-CH','CH-CO','CO-CO'],['N-P','P-C1','C2-C3','C1-C2','C1-C5'],['N-C1-P','P-C2-C1','C1-C3-C2','C2-C4-C3','C2-C5-C1','C6-C1-C5']]    
    if mtype=='dmpc_A5':
        PairNamesList=[['N-N','N-P','N-CH','N-CO','P-P','P-CH','P-CO','CH-CH','CH-CO','CO-CO'],['N-P','P-C1','C2-C3','C1-C2','C1-C5'],['N-C1-P','P-C2-C1','C1-C3-C2','C2-C4-C3','C2-C5-C1']]    
    if mtype=='':
        print 'Automatically defining list of interactions from input file. If this does not work well for your input, edit subroutine GetDFsFromFile_magic in MagicTools.py and manualy add list of your beads to PairNamesList'
        rdffile=open(rdffilename,'r')
        lines=rdffile.readlines()
        l=[]
        for line in lines:
            if re.search('This pair|This triplet',line):
                l.append(line.split('\n')[0].strip())
        s=set(l) # leave only one exemplar of every pair
        ll=[]
        for line in l: #order pairs as in the file
            if line in s:
                ll.append(line)
                s.remove(line)
        PairNamesList=[[],[],[]]
        for line in ll:
            if 'intermolecular' in line:
                PairNamesList[0].append(line.split('This pair:')[1].split('intermolecular')[0].strip().replace(' ',''))                
            if 'intramolecular' in line:
                PairNamesList[1].append(line.split('This pair:')[1].split('intramolecular')[0].strip().replace(' ',''))
            if 'triplet:' in line:
                PairNamesList[2].append(line.split('triplet:')[1].strip().replace(' ',''))
        print len(PairNamesList[0]),' intermolecular pairs detected ',PairNamesList[0]
        print len(PairNamesList[1]),' pairwise bonds detected ',PairNamesList[1]
        print len(PairNamesList[2]),' angles detected ',PairNamesList[2]

    if basename!='':
        basename+=' '
        
    if DF.upper() not in ('RDF','RDFREF','POT','NEWPOT'):
        print 'ERROR: Unknown type of distribution function (DF)!'
        return
    RDFsList=[]    
    name=''
    RMin=0
    RMax=0
    Npoints=0
    RDF=np.zeros((1,2))
    def _GrepText_(DFtype='inter',header='RDF ',column=2, s=None):
                '''
                Parsing set of lines <lines> for  regular expression <s> and returnd object of a proper class (it is always subclass of DF)
                '''
                print 'Pair '+ str(pair)
                RDF_r_List=[]
                RDF_gr_List=[]
                name=basename+pair+' i'+str(irep)
                # search for resulting output (Final output, not a check one)
                for l in lines:
                  if re.search(s,l):
                    RDF_r_List.append(float(l.split()[0]))
                    RDF_gr_List.append(float(l.split()[int(column)-1]))
                RDF=np.array([[RDF_r_List[i],RDF_gr_List[i]] for i in xrange(0,len(RDF_r_List))])
                RMin=RDF[:,0].min()
                RMax=RDF[:,0].max()
                Npoints=len(RDF[:,0])
                resol=(RMax-RMin)/(Npoints-1)
                RMin=RMin-resol/2
                RMax=RMax+resol/2
                for i in xrange(0,Npoints-1):
                    if (abs(RDF[i+1,0]-RDF[i,0]-resol)>=0.0001):
                        print 'Error in the table, points are not equidistant',RMax,RMin,Npoints,resol,RDF[0,i+1],RDF[0,i]
                        return                
                if (DFtype=='intra') and (header=='Pot1-3 '):                    
                    trdf=DF.C_Pot13(Type='A',Name=header+name,Min=RMin,Max=RMax,Npoints=Npoints,g=RDF,
                                     Kind='POTENTIAL')#,resol=resol)
                elif (DFtype=='intra') and (header=='Pot '):
                    trdf=DF.C_PotBond(Type='B',Name=header+name,Min=RMin,Max=RMax,Npoints=Npoints,g=RDF,
                                     Kind='POTENTIAL')#,resol=resol)resol=resol)
                elif (DFtype=='inter') and (header=='Pot '):
                    trdf=DF.C_PotInter(Type='NB',Name=header+name,Min=RMin,Max=RMax,Npoints=Npoints,g=RDF,
                                        Kind='POTENTIAL')#,resol=resol)resol=resol)
                elif (header=='RDFref '):
                    if DFtype=='inter':
                        trdf=DF.RDFref_NB(Type='NB',Name=header+name,Min=RMin,Max=RMax,Npoints=Npoints,g=RDF,
                                           Kind='RDF')#resol=resol)
                    elif DFtype=='intra':                        
                        trdf=DF.RDFref_B(Type='B',Name=header+name,Min=RMin,Max=RMax,Npoints=Npoints,g=RDF,
                                           Kind='RDF')#resol=resol)
                elif (header=='ADFref '):
                        trdf=DF.RDFref_A(Type='A',Name=header+name,Min=RMin,Max=RMax,Npoints=Npoints,g=RDF,
                                           Kind='RDF')#resol=resol)
                elif (header=='RDF '):
                    if DFtype=='inter':
                        trdf=DF.RDF_NB(Type='NB',Name=header+name,Min=RMin,Max=RMax,Npoints=Npoints,g=RDF,
                                           Kind='RDF')#resol=resol)
                    elif DFtype=='intra':                        
                        trdf=DF.RDF_B(Type='B',Name=header+name,Min=RMin,Max=RMax,Npoints=Npoints,g=RDF,
                                           Kind='RDF')#resol=resol)
                elif (header=='ADF '):
                        trdf=DF.RDF_A(Type='A',Name=header+name,Min=RMin,Max=RMax,Npoints=Npoints,g=RDF,
                                           Kind='RDF')#resol=resol)                                           
                else:
                    print "Error: Unknown type of the function!"
                    sys.exit(0)
                    
                return trdf
    
    rdffile=open(rdffilename,'r')
    lines=rdffile.readlines()
    # Get total number of iterations performed:
    if (iters==()):
        TotalNumIter=0
        for l in lines:
            if re.search('Final Corrected effective potential',l):
                TotalNumIter+=1
        print 'Total number of iterations:', TotalNumIter
        iters=range(1,TotalNumIter+1)
    else:
         TotalNumIter=len(iters)
    # Get number of checks for every iteration
    TotalNumCheck=0
    for l in lines:
        if re.search('Global Potential correction convergence check',l):
            TotalNumCheck+=1
    print 'Total number of checks per iteration:', TotalNumCheck/TotalNumIter
    
    for irep in iters: #loop over all iterations
        print 'Iteration %i' %(irep)
        if (DF.upper()=='RDFREF')and(iters.index(irep)>0):
            return(RDFs)
        RDFs=[]                    
        for pair in PairNamesList[0]:                
            if (DF.upper()=='RDFREF')and(iters.index(irep)==0):
                s='pot: *'+pair.split('-')[0].strip()+' *'+pair.split('-')[1].strip()+' *Final *'+str(irep)+' '
                RDFs.append(_GrepText_(DFtype='inter',header='RDFref ',column=3, s=s))
            else:
            #Loop over intermolecular pairs
                s='pot: *'+pair.split('-')[0].strip()+' *'+pair.split('-')[1].strip()+' *Final *'+str(irep)+' '
                if DF.upper()=='RDF':
                    RDFs.append(_GrepText_(DFtype='inter',header='RDF ',column=2,s=s))
                if DF.upper()=='POT':
                    RDFs.append(_GrepText_(DFtype='inter',header='Pot ',column=5,s=s))
                if DF.upper()=='NEWPOT':
                    RDFs.append(_GrepText_(DFtype='inter',header='Pot ',column=4,s=s))
                if DF.upper()=='POTCORR':
                    RDFs.append(_GrepText_(DFtype='inter',header='PotCorr ',column=6,s=s))
        for pair in PairNamesList[1]:
            if (DF.upper()=='RDFREF')and(iters.index(irep)==0):
                s='pot: *'+pair.split('-')[0].strip()+' *'+pair.split('-')[1].strip()+' *'+'[0-9] *Final *'+str(irep)+' '
                RDFs.append(_GrepText_(DFtype='intra',header='RDFref ',column=3, s=s))
            else:
            #Loop over intramolecular pairs with pairwise bonds
                s='pot: *'+pair.split('-')[0].strip()+' *'+pair.split('-')[1].strip()+' *'+'[0-9] *Final *'+str(irep)+' '
                if DF.upper()=='RDF':
                    RDFs.append(_GrepText_(DFtype='intra',header='RDF ',s=s))
                if DF.upper()=='POT':
                    RDFs.append(_GrepText_(DFtype='intra',header='Pot ',column=5,s=s))
                if DF.upper()=='NEWPOT':
                    RDFs.append(_GrepText_(DFtype='intra',header='Pot ',column=4,s=s))
                if DF.upper()=='POTCORR':
                    RDFs.append(_GrepText_(DFtype='intra',header='PotCorr ',column=6,s=s))
        for pair in PairNamesList[2]:
            if (DF.upper()=='RDFREF')and(iters.index(irep)==0):
                s='pot: *'+pair.split('-')[0].strip()+' *'+pair.split('-')[1].strip()+' *'+pair.split('-')[2].strip()+' *'+'[0-9] *Final *'+str(irep)+' '
                RDFs.append(_GrepText_(DFtype='intra',header='ADFref ',column=3,s=s))
            else:
                #Loop over intramolecular triplets with 1-3 bonds
                s='pot: *'+pair.split('-')[0].strip()+' *'+pair.split('-')[1].strip()+' *'+pair.split('-')[2].strip()+' *'+'[0-9] *Final *'+str(irep)+' '
                if DF.upper()=='RDF':
                    RDFs.append(_GrepText_(DFtype='intra',header='ADF ',s=s))
                if DF.upper()=='POT':
                    RDFs.append(_GrepText_(DFtype='intra',header='Pot1-3 ',column=5,s=s))
                if DF.upper()=='NEWPOT':
                    RDFs.append(_GrepText_(DFtype='intra',header='Pot1-3 ',column=4,s=s))                    
                if DF.upper()=='POTCORR':
                    RDFs.append(_GrepText_(DFtype='intra',header='PotCorr1-3 ',column=6,s=s))
        RDFsList.append(RDFs)
    return RDFsList

#def GetPotsFromFile_pot(rdffilename,iters=(),DF='RDF',mtype='',basename='',mcmfile=None):
#    '''
#    Analyse pot-file <potfilename> produced by MAGIC and return a list of potentials.
#    basename - prefix given to all the potentials read from the file
#    mcmfile - optional file (or list of files) to read atom names, if not stated a warning will be givean, and atom numbers will be stated instead of types
#    Ucut - Criteria to cut artificial core part of the potential.
#
#    '''
#    print 'Function GetPotsFromFile_pot is obsolete in MagiC 2.0. Please consider using the function ReadPot instead.'
#    print 'If you still want to proceed, use __GetPotsFromFile_pot '    
#    return
    
def __GetPotsFromFile_pot(potfilename,basename='',mcmfile='',Ucut=10000.0):    
    '''
    Analyse pot-file <potfilename> produced by MAGIC and return a list of potentials.
    basename - prefix given to all the potentials read from the file
    mcmfile - optional file (or list of files) to read atom names, if not stated a warning will be givean, and atom numbers will be stated instead of types
    Ucut - Criteria to cut artificial core part of the potential.
    '''    
    PotList=[]
    potfile=open(potfilename,'r')
    lines=potfile.readlines()
    potfile.close()
    line=lines.pop(0)
    NAtomTypes=int(line.split()[0])
    NPoints0=int(line.split()[1])
    NInter=int(line.split()[2])
    NIntra=int(line.split()[3])
    N1_3=int(line.split()[4])    
    RMin0=float(line.split()[5])
    RMax0=float(line.split()[6])
# Reading MCM file
    if mcmfile=='': # No mcm-file provided
        print 'No mcm file given, use numbers instead of names for atoms and atomic types'
        GotTypeNames=False
        GotAtomNames=False
    else: # an mcm-file or a list of mcm-files provided
        if not isinstance(mcmfile,type([])): # single file-name provided - convert to a list
            mcmfile=[mcmfile]
        AtomsName={}
        AtomsCharge=[]
        AtomsType=[]
        AtomTypes={}  
        AtomTypesCharge={} # Charge of the given atom type
        AtomsTypeName=[]
        NAtoms=0 #Total number of atoms in all mcm-files        
        for imcmfile in mcmfile: # over all mcm-files in the list
            try:
                mcm=open(imcmfile,'r')
            except:
                print 'Error while opening file {0}'.format(imcmfile)
                break
            mcmlines=[] # drop the memory if it exists
            mcmlines=mcm.readlines()
            for i in mcmlines:
                if i.startswith('#'):
                    mcmlines.remove(i)
            mcmline=mcmlines.pop(0).split('\n')[0]        
            iNAtoms=int(mcmline)
            for i in xrange(0,iNAtoms):
                AtomsName[i+1+NAtoms]=mcmlines[i].split()[0]    
            GotAtomNames=True
            
            AtomsType+=[int(mcmlines[i].split()[6]) for i in xrange(0,iNAtoms)] # Number of atomic type of the given atom
            AtomsCharge+=[float(mcmlines[i].split()[5]) for i in xrange(0,iNAtoms)] # Charge of given atom
            GotTypeNames=True
            try:
                AtomsTypeName+=[mcmlines[i].split()[7] for i in xrange(0,iNAtoms)]
            except:
                print 'Error while reading names of the atomic types in the mcm-file {0}. Check if they are stated there.'.format(imcmfile)
                GotTypeNames=False            
            NAtoms+=iNAtoms
        for i in xrange(0,NAtoms):
            AtomTypesCharge[AtomsType[i]]=AtomsCharge[i]
            AtomTypes[AtomsType[i]]=AtomsTypeName[i]

# Reading Intermolecular potentials       
    if NAtomTypes>0:
        resol=(RMax0-RMin0)/NPoints0
    for iNInter in xrange(0,NInter):                
        NPoints=NPoints0
        RMin=RMin0
        RMax=RMax0        
        while float(lines[0].split('\n')[0].split()[1])>Ucut:             
            line=lines.pop(0).split('\n')[0]
            RMin+=resol            
            NPoints-=1
        RDFs=np.zeros((NPoints,2))
        for i in xrange(0,NPoints):                                                    
            line=lines.pop(0).split('\n')[0]
            r=float(line.split()[0])
            g_r=float(line.split()[1])
            ai=int(line.split()[2])
            aj=int(line.split()[3])
            ind=int((r-RMin)/resol)
            RDFs[ind,0]=r
            RDFs[ind,1]=g_r             
        #Creating DF.PotInter - object:
        if GotTypeNames:
            tpot=DF.Pot_NB(Type='NB',Name=basename+'Pot '+AtomTypes[ai]+'-'+AtomTypes[aj],
                            Min=RMin,Max=RMax,Npoints=NPoints,
                            g=RDFs[:,:],
                            AtomTypes=[AtomTypes[ai],AtomTypes[aj]])            
            tpot.qq=AtomTypesCharge[ai]*AtomTypesCharge[aj]
        else:
            tpot=DF.Pot_NB(Type='NB',Name=basename+'Pot AtomType'+str(ai)+'-AtomType'+str(aj),
                            Min=RMin,Max=RMax,Npoints=NPoints,g=RDFs[:,:],AtomTypes=[str(ai),str(aj)])                                
        PotList.append(tpot)
# Reading Intramolecular pair potentials
    for i in xrange(0,NIntra):
        line=lines.pop(0).split('\n')[0]
        npairs=int(line.split()[0])
        ai=int(line.split()[1])
        aj=int(line.split()[2])
        NPoints=int(line.split()[3])
        RMin=float(line.split()[4])
        RMax=float(line.split()[5])
        resol=(RMax-RMin)/NPoints
        groups=[]
        for i in xrange(0,npairs):
            line=lines.pop(0)
            groups.append(map(int,line.strip().split()))            
            
        RDFs=np.zeros((NPoints,2))
        for j in xrange(0,NPoints):
            line=lines.pop(0).split('\n')[0]
            r=float(line.split()[0])
            g_r=float(line.split()[1])
            ind=int((r-RMin)/resol)
            RDFs[ind,0]=r
            RDFs[ind,1]=g_r 
        #Creating DF.PotBond - object
        if GotAtomNames:
            tpot=DF.Pot_B(Type='B',Name=basename+'Pot '+AtomsName[ai]+'-'+AtomsName[aj],
                           Min=RMin,Max=RMax,Npoints=NPoints,g=RDFs[:,:],AtomGroups=groups)
        else:
            tpot=DF.Pot_B('B',Name=basename+'Pot Atom'+str(ai)+'-Atom'+str(aj),
                           Min=RMin,Max=RMax,Npoints=NPoints,g=RDFs[:,:],AtomGroups=groups)
        PotList.append(tpot)
        
# Reading Intramolecular 1-3 angle bending potentials
    for i in xrange(0,N1_3):
        line=lines.pop(0).split('\n')[0]
        npairs=int(line.split()[0])
        ai=int(line.split()[1])
        aj=int(line.split()[2])
        ak=int(line.split()[3])
        NPoints=int(line.split()[4])
        RMin=float(line.split()[5])
        RMax=float(line.split()[6])
        resol=(RMax-RMin)/NPoints
        groups=[]
        for i in xrange(0,npairs):
            line=lines.pop(0)
            groups.append([int(line.strip().split()[i]) for i in [0,2,1]])            
        RDFs=np.zeros((NPoints,2))
        for j in xrange(0,NPoints):
            line=lines.pop(0).split('\n')[0]
            r=float(line.split()[0])
            g_r=float(line.split()[1])
            ind=int((r-RMin)/resol)
            RDFs[ind,0]=r
            RDFs[ind,1]=g_r 
        #Creating DF.Pot13 - object
        if GotAtomNames:
            tpot=DF.Pot_A(Type='A',Name=basename+'Pot13 '+AtomsName[ai]+'-'+AtomsName[ak]+'-'+AtomsName[aj],
                             Min=RMin,Max=RMax,Npoints=NPoints,g=RDFs[:,:],AtomGroups=groups)
        else:
            tpot=DF.Pot_A(Type='A',Name=basename+'Pot13 Atom'+str(ai)+'-Atom'+str(ak)+'-Atom'+str(aj),
                             Min=RMin,Max=RMax,Npoints=NPoints,g=RDFs[:,:],AtomGroups=groups)
        PotList.append(tpot)
    return PotList
    
def tpr2mmol(tprfile):
    '''
    Simple tool which converts GROMACS's tpr file to a number of mmol-files, i.e. 
    files containing lists of atoms with their names, coordinates, masses and charges.
    
    Input tprfile
    '''
    import re        
    import os
    
#    import MolType
#    import Atom

    tprfiledump=tprfile+'.dmp'
    os.system('gmxdump -s {0} > {1}'.format(tprfile, tprfiledump))
    
    ifile=open(tprfiledump,'r')
    lines=ifile.readlines()
    ifile.close()
    
    MTlines=[line.strip() for line in lines if re.search('moltype +=',line)!=None]
    MTNames=[re.search('".*"',line).group().replace('"','') for line in MTlines]
    
    NMollines=[line.strip() for line in lines if re.search('#molecules +=',line)!=None]
    NMols=[int(line.split('=')[1]) for line in NMollines]
    
    NAtomlines=[line.strip() for line in lines if re.search('#atoms_mol +=',line)!=None]
    NAtoms=[int(line.split('=')[1]) for line in NAtomlines]
    
    lBegMT=[i for iMT, MT in enumerate(MTNames) for i,l in enumerate(lines) if 'moltype ({0:<d})'.format(iMT) in l]
    lBegMT.append(len(lines)-1)
    
    Rlines=[line.strip() for line in lines if re.search(' +x\[ *[0-9]*\]=\{.*\}',line)!=None]
    R=[[float(rec)*10.0 for rec in line.replace('}','').split(']={')[1].split(',')] for line in Rlines ]
    rBeg=0
    for iMT, MT in enumerate(MTNames):            
        ll=lines[lBegMT[iMT]:lBegMT[iMT+1]]
        ANamelines=[line.strip() for line in ll if re.search('atom\[[0-9]*\]=\{name=.*\}',line)!=None]
        ANames=[re.search('".*"',line).group().replace('"','') for line in ANamelines]
        AProplines=[line.strip() for line in ll if re.search('atom\[ *[0-9]*\]=\{type=.*\}',line)!=None]
        AProps=[[[rec.split('=')[1] for rec in line.split(']={')[1].split(',')][i] for i in [0,3,4] ] for line in AProplines]#[float(rec.split('=')[1].strip()[i]) for line in AProplines for rec in line.split(']={')[1].split(',') for i in [0,3,4]]
        Ri=R[rBeg:rBeg+NAtoms[iMT]]
        rBeg+=NMols[iMT]*NAtoms[iMT]

        # instead of creating a moltype, I just write a mmol file, which is easier for now
#        MType=MolType.MolType(MT,type='dump')
#        MType.Atoms=[Atom.Atom(ANames[i],Ri[i][0],Ri[i][1],Ri[i][2],AProps[i][1],AProps[i][2], TypeID=AProps[i][0]+1) for i in xrange(0,NAtoms[iMT])]        
#        MType.NAtom=len(MType.Atoms)
#        MType.WriteMCM()
        ofile=open(MT+'.mmol','w')    
        ofile.write('{0}\n'.format(NAtoms[iMT]))
        ofile.write('# {0} {1} {2} {3} {4} {5} \n'.format('Atom','X', 'Y', 'Z', 'Mass', 'Charge'))
        ofile.writelines(['{0} {1} {2} {3} {4} {5} \n'.format(ANames[i], 
                          Ri[i][0], Ri[i][1], Ri[i][2], AProps[i][1], 
                            AProps[i][2]) for i in xrange(NAtoms[iMT])])
        ofile.write('\n')
        ofile.close()
        
def gro2xmol(ifilename, ofilename='output.xmol'):
    import sys
    #import pdb
                  
    ifile=open(ifilename,'r')
    ofile=open(ofilename,'w')
    templine=ifile.readline()
    templine=ifile.readline()
    ifile=open(ifilename,'r')
    natomsinp=int(templine.split('\n')[0])
    nlinestoread=int(templine.split('\n')[0])+3
    lines=[ifile.readline() for iline in xrange(0,nlinestoread)]
    while lines[nlinestoread-1]!='':
        ll=[]
        for i in xrange(0,len(lines)):
            ll.append(lines[i].split('\n')[0])
        natoms=int(ll.pop(1).lstrip())
        if natoms!=natomsinp:
            print 'Bad input: Total number of atoms in the system is different from sum(nmols*natimol)'
            sys.exit()
        box=[float(i)*10.0 for i in ll[natoms+1].split()]
        if 't=' in ll[0]:
            t=float(ll.pop(0).split('t=')[1])
        else :
            templine=ll.pop(0)
            t=0.0
        ofile.write('%10i \n' %(natoms)) 
        ofile.write(' after %12.4f fs, BOX: %8.4f    %8.4f    %8.4f \n' %(t*1000.0,box[0], box[1], box[2]))
            
        atomname=[l.split()[1].lstrip() for l in ll[0:natoms]]
        X=[float(l.split()[3].lstrip())*10.0 for l in ll[0:natoms]] #in nm
        Y=[float(l.split()[4].lstrip())*10.0 for l in ll[0:natoms]] #nm
        Z=[float(l.split()[5].lstrip())*10.0 for l in ll[0:natoms]] #nm
        i=0
        
        for i in xrange(0,len(atomname)):
            ofile.write('%4s %7.3f %7.3f %7.3f \n' %(atomname[i],X[i],Y[i],Z[i]))
        lines=[ifile.readline() for iline in xrange(0,nlinestoread)]
        
    ofile.close()
    ifile.close()

def xmol2gro(ifilename, ofilename, molnames, nmols, natimol, nconf=1):    
    ''' 
    Converts the xmol-file into gro-file.
    Example for two component system of 98 molecules of DMPC (118 atoms) and 2700 molecules of water (3 atoms)
    xmol2gro('input.xmol','output.gro',['DMPC','WAT'], [98,2700], [118,3],nconf=1)
        '''

    import sys
    import pdb
          
    ifile=open(ifilename,'r')
    ofile=open(ofilename,'w')
    for iconf in xrange(0,nconf):
        lines=[]
        for i in xrange(0,sum([nmols[i]*natimol[i] for i in xrange(0,len(nmols))])+2):
            lines.append(ifile.readline().split('\n')[0])
        ll=lines
        if len(nmols)!=len(natimol):
            print 'Bad input len(nmols)!=len(natimol)'
            sys.exit()
        natoms=int(ll.pop(0).lstrip())
        if natoms!=sum([nmols[i]*natimol[i] for i in xrange(0,len(nmols))]):
            print 'Bad input: Total number of atoms in the system is different from sum(nmols*natimol)'
            sys.exit()
        box=[float(i) for i in ll[0].split('BOX:')[1].split()]
        ofile.write('DMPC Lipid geometry converted from MDynamix xmol\n')
        ofile.write(str(natoms)+'\n') 
    #    molname='DMPC'
        atomname=[l.split()[0].lstrip() for l in ll[1:natoms+1]]
        X=[float(l.split()[1].lstrip())*0.1 for l in ll[1:natoms+1]] #in nm
        Y=[float(l.split()[2].lstrip())*0.1 for l in ll[1:natoms+1]] #nm
        Z=[float(l.split()[3].lstrip())*0.1 for l in ll[1:natoms+1]] #nm
        i=0
        for inmol in xrange(0,len(nmols)): #over types
            for  imol in xrange(0,nmols[inmol]): #over molecules of type
                for iat in xrange(0,natimol[inmol]): #over all atoms ov molecule of given type
                    ofile.write("%5i%5s%5s%5i%8.3f%8.3f%8.3f" %(imol+1,molnames[inmol].rjust(4),atomname[i].rjust(5),i+1,X[i],Y[i],Z[i])+'\n')
                    i+=1
        ofile.write("%8.5f %8.5f %8.5f\n" %(box[0]*0.1,box[1]*0.1,box[2]*0.1))
    ofile.close()