!   Module Forces
!   Subroutines for calculation of energies and forces
!
!   ENERGY - calculation of total energy with looping over molecules (should be more effective)
!   ENERG - energy and force for a non-bonded pair of atoms
!   ENERI - energy and force for a bonded pair of atoms
!   ENERI_ANG - energy and force for a  pair of atoms bonded with bending angle bond
!   EWALD - reciprocal space Ewald contribution to energy
!   INIFUR - initialization of Ewald sum
!   DIFEW - change of energy from recipocal Ewald due to change of 1 particle
!   DIFEWM -Change of reciprocal Ewald energy if 1 molecule change position1
!   RECSIN - recalculation of arrays for Ewald in the case of accepted step for 1 atom
!   RECSINM - recalculation of arrays for Ewald in the case of accepted step for 1 molecule
!remove   FOIRIERF - contribution to forces from Ewald summation
!
!DEC$ FREEFORM
module modforces

use utilsmod
integer(4),private,parameter:: NPOTMP=4000
integer(4),private:: inifurdone=0
real(8),allocatable,private :: DSIN(:),DCOS(:)
real(8),allocatable,private ::sigmasin(:), sigmacos(:)  !
real(8), parameter :: TOTPI = 1.128379169D0
real(8) :: RAD2DEG=180.d0/3.1415926535897932d0
contains

!========================= DIFEW ================================
!
!  Change of reciprocal Ewald energy if I-th particle change
!  position to X1,Y1,Z1
!
      subroutine DIFEW(X1,Y1,Z1,I,CHI,DEW,RX,RY,RZ,RKV,SSIN,SCOS,NKV,SX,SY,SZ)
! Global variables
        real(8) X1,Y1,Z1,CHI,DEW
        real(8),allocatable::RX(:),RY(:),RZ(:),RKV(:),SSIN(:),SCOS(:),SX(:),SY(:),SZ(:)
        integer(4) I,NKV
    ! Local variables
        real(8) RKX,RKY,RKZ,SCO,SCN,SCP,SINR,DSN,DCS,DDD
        integer(4) IKV
!   reciprocal space Evald
      DEW=0.d0
      CHI=2.d0*CHI
      do IKV=1,NKV      
        RKX=RX(IKV)
        RKY=RY(IKV)
        RKZ=RZ(IKV)
        SCO=0.5*(RKX*SX(I)+RKY*SY(I)+RKZ*SZ(I))
        SCN=0.5*(RKX*X1+RKY*Y1+RKZ*Z1  )
	SCP=SCO+SCN
	SINR=sin(SCN-SCO)*CHI
        DSN=cos(SCP)*SINR
        DCS=-sin(SCP)*SINR
        DSIN(IKV)=DSN
        DCOS(IKV)=DCS
        DDD=((2.d0*SSIN(IKV)+DSN)*DSN+(2.d0*SCOS(IKV)+DCS)*DCS)*RKV(IKV)
        DEW=DEW+DDD
      end do
      return
      end  subroutine DIFEW
!========================= DIFEWM ================================
!
!  Change of reciprocal Ewald energy if "I" molecule, represented by list of charged atoms ichatomlist change
!  position
!
subroutine DIFEWM(ichatomlist,nchatom,RX,RY,RZ,RKV,SSIN,SCOS,SX,SY,SZ,NKV,RN,Q,DEW)
!Global variables
    integer(4),allocatable:: ichatomlist(:,:) ! list of charged atoms of the shifted molecule
    integer(4) NKV,nchatom ! Number of charged shifted atoms
    real(8) DEW ! Change in reciprocal Ewald energy because of shift
    real(8),allocatable::RX(:),RY(:),RZ(:),RKV(:),SSIN(:),SCOS(:),SX(:),SY(:),SZ(:),Q(:),RN(:,:) ! new trial coordinates of charged shifted atoms    
! local variables
    integer i,j,inkv,II
    real(8):: deltae=0.d0

!DEBUG		!write(*,*) 'Inner: Ich,Q,    Xo,     Yo,     Zo,     Xn,    Yn,    Zn'
    do i=1,nchatom
        II=ichatomlist(i,2) ! Global atomic number of i-th charged atom
	j=ichatomlist(i,1) ! Number in the molecule of i-th charged atom
!DEBUG	  	!	write(*,'(a,i,7f9.4)') 'inner:',i,Q(II),SX(II),SY(II),SZ(II),RN(1,j),RN(2,j),RN(3,j)
    enddo
    deltae=0.d0
    do inkv=1,NKV ! over all K-vectors
        sigmasin(inkv)=0.d0 ! Check if this line can be removed later
	sigmacos(inkv)=0.d0 ! Check if this line can be removed later
	do i=1,nchatom
            II=ichatomlist(i,2) ! Global atomic number of i-th charged atom
	    j=ichatomlist(i,1) ! Number in the molecule of i-th charged atom
            sigmacos(inkv)=sigmacos(inkv)+Q(II)*(cos(RX(inkv)*RN(1,j)+RY(inkv)*RN(2,j)+RZ(inkv)*RN(3,j))&
                            -cos(RX(inkv)*SX(II)+RY(inkv)*SY(II)+RZ(inkv)*SZ(II)))
            sigmasin(inkv)=sigmasin(inkv)+Q(II)*(sin(RX(inkv)*RN(1,j)+RY(inkv)*RN(2,j)+RZ(inkv)*RN(3,j))&
	  				-sin(RX(inkv)*SX(II)+RY(inkv)*SY(II)+RZ(inkv)*SZ(II)))
	enddo
	deltae=deltae+(sigmacos(inkv)*(2.0*SCOS(inkv)+sigmacos(inkv))+sigmasin(inkv)*(2.0*SSIN(inkv)+sigmasin(inkv)))*RKV(inkv)
    enddo
    DEW=deltae				!
end subroutine DIFEWM

!
!=================== RECSIN ========================================
!  Update of arrays SSIN,SCOS used in Ewald summation
!  this procedure must be called if the singe atom MCstep is accepted
!
subroutine RECSIN(NKV,SSIN,SCOS)
    integer(4) NKV
    real(8),allocatable::SSIN(:),SCOS(:)
    integer(4) IKV
    do IKV=1,NKV
        SSIN(IKV)=SSIN(IKV)+DSIN(IKV)
        SCOS(IKV)=SCOS(IKV)+DCOS(IKV)
    end do
end subroutine RECSIN
!=================== RECSINM ========================================
!   Update of arrays SSIN,SCOS used in Ewald summation
!  this procedure must be called if the MC step for the whole molecule is accepted
!
subroutine RECSINM(NKV,SSIN,SCOS)
    integer(4) NKV
    real(8),allocatable::SSIN(:),SCOS(:)
    integer(4) IKV
    do IKV=1,NKV
        SSIN(IKV)=SSIN(IKV)+sigmasin(IKV)
        SCOS(IKV)=SCOS(IKV)+sigmacos(IKV)
    end do
end subroutine RECSINM
!!========== INIFUR ===============================================
! Initialize Fourier
subroutine INIFUR(HBOXL,HBOYL,HBOZL,BOXL,BOYL,BOZL,VOL,ALPHA,AF,RECUT,FQ,&
                        COULF,RX,RY,RZ,RKV,ESF,FKJM,NKV,NSTOT,Q,ID)
    real(8) HBOXL,HBOYL,HBOZL,BOXL,BOYL,BOZL,VOL,ALPHA,AF,RECUT,FQ,COULF,ESF,FKJM
    real(8), allocatable:: RX(:),RY(:),RZ(:),RKV(:),Q(:)
    integer(4) NKV,NSTOT,ID
    include "dimpar.h"

    real(8) CUTK,CUTK2,FKX,FKY,FKZ,CFUR,RK2,AFF
    integer(4) KMAX,KMAY,KMAZ,IKV,IKZ,IKX,IKY,KY1,KY2,I
    inifurdone=1
    HBOXL= 0.5*BOXL
    HBOYL= 0.5*BOYL
    HBOZL= 0.5*BOZL
    VOL	= BOXL*BOYL*BOZL
#ifdef PBC_TO
    VOL=VOL*0.5
#endif
    ALPHA=AF/RECUT
    CUTK2=4.d0*ALPHA**2*FQ
    CUTK=sqrt(CUTK2)
    FKX=2.d0*PI/BOXL
    FKY=2.d0*PI/BOYL
    FKZ=2.d0*PI/BOZL
    KMAX=CUTK/FKX+1
    KMAY=CUTK/FKY+1
    KMAZ=CUTK/FKZ+1
    IKV=0
    CFUR=COULF/(VOL*PI)
    do IKZ=-KMAZ,KMAZ
        do IKX=0,KMAX
            if(IKX.eq.0)then
                if(IKZ.gt.0)then
                    KY1=0
                else
                    KY1=1
                end if
                KY2=KMAY
            else
                KY1=-KMAY
                KY2=KMAY
            end if
            do IKY=KY1,KY2
                RK2=(IKX*FKX)**2+(IKY*FKY)**2+(IKZ*FKZ)**2
                if(RK2.le.CUTK2)then
#ifdef PBC_TO
                    if (abs(mod(IKX+IKY+IKZ,2)).eq.0) IKV=IKV+1 ! For truncated octahedron we consider only Kvec with even sum of coordinates, odd will be zero
#else
                    IKV=IKV+1
#endif
                end if
            end do
        end do
    end do
    if (.not. allocated(RX)) then 
       allocate(RX(IKV),RY(IKV),RZ(IKV),RKV(IKV))
       RX(:)=0.d0
       RY(:)=0.d0
       RZ(:)=0.d0
       RKV(:)=0.d0
    endif
    IKV=0
    do IKZ=-KMAZ,KMAZ
        do IKX=0,KMAX
            if(IKX.eq.0)then
                if(IKZ.gt.0)then
                    KY1=0
                else
                    KY1=1
                end if
                KY2=KMAY
            else
                KY1=-KMAY
                KY2=KMAY
            end if
            do IKY=KY1,KY2
                RK2=(IKX*FKX)**2+(IKY*FKY)**2+(IKZ*FKZ)**2
#ifdef PBC_TO
                if((RK2.le.CUTK2).and.(abs(mod(IKX+IKY+IKZ,2).eq.0)))then
#else
                if(RK2.le.CUTK2)then
#endif
                    IKV=IKV+1
                    RX(IKV)=IKX*FKX
                    RY(IKV)=IKY*FKY
                    RZ(IKV)=IKZ*FKZ
                    RKV(IKV)=4.*PI**2*CFUR*dexp(-0.25*RK2/ALPHA**2)/RK2
                end if
            end do
        end do
    end do
    NKV=IKV
    write((ID+6),*)'Kmax=',KMAX,'    Num. of k-vectors ',NKV
    ESF=0.
    AFF=ALPHA/sqrt(PI)
    do I=1,NSTOT
        ESF=ESF-AFF*COULF*Q(I)**2
    end do
    write((ID+6),*)' el. self-energy ',ESF*FKJM
    return
end subroutine INIFUR
!!!======================= ENERI ===============================
!
real(8) function ENERI(RR,IPP,FORCE,R1,R2,PotB_riBond,RDINC,NAI,ID)
    real(8) RR,R1,R2,FORCE
    real(8),allocatable::PotB_riBond(:,:)
    integer(4) IPP,ID
    integer(4),allocatable::NAI(:)

    real(8) RD,RDINC,AL,AL1,POT1,POT2, RR1
    integer(4) NAM,NR1,NR2
    integer(4) :: ICNT=0
    RD=R2-R1
    NAM=NAI(IPP)
    RDINC=RD/NAM
    if(NAM.le.1)then
        ENERI=0.
        FORCE=0.
        return
    end if
    if(RR.le.R1)then
        ENERI=PotB_riBond(1,IPP)
        write((ID+6),'(a,2f10.4,i4)')' too short distannce in ENERI:',RR,R1,IPP
        return
    end if
    if(RR.gt.R2)then
        ENERI=PotB_riBond(NAM,IPP)
        write((ID+6),'(a,2f10.4,i4)') ' too large distannce in ENERI:',RR,R2,IPP
        return
    end if
    NR1=(RR-R1)/RDINC+0.5
    if(NR1.le.0)NR1=NR1+1
    NR2=NR1+1
    if(NR2.gt.NAM)then
        NR2=NR2-1
        NR1=NR1-1
    end if
    RR1=R1+(NR1-0.5)*RDINC
    AL=(RR-RR1)/RDINC
    AL1=1.-AL
    POT1=PotB_riBond(NR1,IPP)
    POT2=PotB_riBond(NR2,IPP)
    ENERI=AL1*POT1+AL*POT2
        FORCE=(POT2-POT1)/RDINC
    return
end function ENERI
!
!!!!======================= ENERI_ang ===============================
!
real(8) function ENERI_ang(RR,IPP,FORCE,R1,R2,PotB_riBond,RDINC,NAI,ID)
    real(8) RR,R1,R2,FORCE
    real(8),allocatable::PotB_riBond(:,:)
    integer(4) IPP,ID
    integer(4),allocatable::NAI(:)


    real(8) RD,RDINC,AL,AL1,POT1,POT2, RR1
    integer(4) NAM,NR1,NR2
    integer(4) :: ICNT=0
    RD=R2-R1
    NAM=NAI(IPP)
    RDINC=RD/NAM
    if(NAM.le.1)then
        ENERI_ang=0.
        FORCE=0.
        return
    end if
    if(RR.le.R1)then
        ENERI_ang=PotB_riBond(1,IPP)
        write((ID+6),'(a,2f10.4,i4)')' too small angle in ENERI_ang:',RR,R1,IPP
        return
    end if
    if(RR.gt.R2)then
        ENERI_ang=PotB_riBond(NAM,IPP)
        write((ID+6),'(a,2f10.4,i4)') ' too large angle in ENERI_ang:',RR,R2,IPP
        return
    end if
    NR1=(RR-R1)/RDINC+0.5
    if(NR1.le.0)NR1=NR1+1
    NR2=NR1+1
    if(NR2.gt.NAM)then
        NR2=NR2-1
        NR1=NR1-1
    end if
    RR1=R1+(NR1-0.5)*RDINC
    AL=(RR-RR1)/RDINC
    AL1=1.-AL
    POT1=PotB_riBond(NR1,IPP)
    POT2=PotB_riBond(NR2,IPP)
    ENERI_ang=AL1*POT1+AL*POT2
    Force=0
    return
    end function ENERI_ang
!
!======================= ENERG ===============================
!
real(8) function ENERG(RR,IT,JT,FORCE,NA,RMAX,RDFINC,PotNB_riSjS,ID)
    real(8) RR,FORCE,RMAX,RDFINC
    real(8), allocatable::PotNB_riSjS(:,:,:)
    integer(4) IT,JT,NA,ID

    real(8) AL,AL1,POT1,POT2, RR1
    integer(4) :: ICNT=0,NR1,NR2
    NR1=RR*NA/RMAX+0.5
    if(NR1.le.0)NR1=NR1+1
    NR2=NR1+1
    if(NR2.gt.NA)then
        NR2=NR2-1
        NR1=NR1-1
        if(NR2.gt.NA)then
            write((ID+6),*)' distance > cutoff in ENERG:',RR
            write((ID+6),*) 'NR2:',NR2,' NA:',NA,' IT:',IT,' JT:',JT,' RR:',RR
            stop
        end if
    end if
    RR1=(NR1-0.5)*RMAX/NA
    AL=(RR-RR1)/RDFINC
    AL1=1.-AL
    POT1=PotNB_riSjS(NR1,IT,JT)
    POT2=PotNB_riSjS(NR2,IT,JT)
    ENERG=AL1*POT1+AL*POT2
        FORCE=(POT2-POT1)/RDFINC
    return
end function ENERG
!=============== EWALD =======================================
!
SUBROUTINE EWALD(EFUR,RX,RY,RZ,SX,SY,SZ,Q,SSIN,SCOS,RKV,HBOXL,HBOYL,HBOZL,&
				BOXL,BOYL,BOZL,VOL,ALPHA,AF,RECUT,FQ,COULF,ESF,FKJM,NKV,NCHRG,ICHRG,NSTOT,ID)
    integer(4) NKV,NCHRG,ID
    real(8) EFUR
    real(8),allocatable::RX(:),RY(:),RZ(:),RKV(:),SSIN(:),SCOS(:),SX(:),SY(:),SZ(:),Q(:)
    ! Variables for INIFUR
    real(8) HBOXL,HBOYL,HBOZL,BOXL,BOYL,BOZL,VOL,ALPHA,AF,RECUT,FQ,COULF,ESF,FKJM
    integer(4) NSTOT
    integer(4),allocatable::ICHRG(:)

    integer(4) :: IKV,IA,I,INIT=0
    real(8) RXV,RYV,RZV,SCS,SSN,SC,SS1,CC1
!  Initialisation
    if(inifurdone.eq.0) call INIFUR(HBOXL,HBOYL,HBOZL,BOXL,BOYL,BOZL,VOL,ALPHA,AF,&
					RECUT,FQ,COULF,RX,RY,RZ,RKV,ESF,FKJM,NKV,NSTOT,Q,ID)
    if (.not. allocated(SSIN)) then 
        allocate(SSIN(NKV),SCOS(NKV),DSIN(NKV),DCOS(NKV),sigmasin(NKV),sigmacos(NKV))
        SSIN(:)=0.d0
        SCOS(:)=0.d0
        DSIN(:)=0.d0
        DCOS(:)=0.d0
        sigmasin(:)=0.d0
        sigmacos(:)=0.d0
    endif
    INIT=1
    EFUR=0.
!   reciprocal space Evald
    do IKV=1,NKV
        RXV=RX(IKV)
        RYV=RY(IKV)
        RZV=RZ(IKV)
        SCS=0.
        SSN=0.
!$DIR NO_RECURRENCE
        do IA=1,NCHRG
            I = ICHRG(IA)
            SC=(RXV*SX(I)+RYV*SY(I)+RZV*SZ(I))
	    SS1=Q(I)*sin(SC)
	    CC1=Q(I)*cos(SC)
            SSN=SSN+SS1
            SCS=SCS+CC1
        end do
	SSIN(IKV)=SSN
	SCOS(IKV)=SCS
        EFUR=EFUR+RKV(IKV)*(SSN**2+SCS**2)
    end do
    return
end subroutine EWALD
!
!
!===================== ENERGY ========================================
!
    subroutine ENERGY(SX,SY,SZ,RCUT,RECUT,Q,COULF,ALPHA,RMN,RMX,EEL,ESF,&
                              EFUR,ENER,DE,EINT,EINT_ang,NSTOT,ITYPE,STypeAtom,NNUM,SiteAtom,Bond_iSjS,LMoveMType,&
                              NKV,NCHRG,ICHRG,RX,RY,RZ,SSIN,SCOS,RKV,&
                              HBOXL,HBOYL,HBOZL,BOXL,BOYL,BOZL,VOL,AF,FQ,FKJM,&
                              RMAX,RDFINC,RDINC,PotNB_riSjS,PotB_riBond,&
                              NA,&
                              !IE,
                              NAI,IPRINT,IND,ID,NII,NII_ang,BTypeBond,&
                              NOP,AdrAtomMol,NSiteMType,MTypeMol,NPTT,num_mt_ang_bond_types,mt_ang_bonds)
    implicit none                              
    !------Global Variables--------------------
    real(8) RCUT,RECUT,COULF,ALPHA,EEL,ESF,EFUR,ENER,DE,EINT,EINT_ang
    real(8),allocatable::RMX(:),RMN(:),SX(:),SY(:),SZ(:),Q(:)
    integer(4) NSTOT,IPRINT,ID,NOP
    integer(4), allocatable::AdrAtomMol(:),NSiteMType(:),MTypeMol(:),ITYPE(:),&
                        STypeAtom(:),NNUM(:),SiteAtom(:),ICHRG(:),Bond_iSjS(:,:),IND(:,:,:)
    logical LMoveMType(:)    
    !------Global Variables-trusted-------------------
    integer(4), allocatable:: num_mt_ang_bond_types(:,:),MT_ang_bonds(:,:,:,:),NPTT(:), BTypeBond(:)
    integer(4) NII,NII_ang
    ! - Variables for EWALD
    integer(4) NKV,NCHRG
    real(8),allocatable:: RX(:),RY(:),RZ(:),RKV(:),SSIN(:),SCOS(:)
    ! Variables for INIFUR
    real(8) HBOXL,HBOYL,HBOZL,BOXL,BOYL,BOZL,VOL,AF,FQ,FKJM
    ! Variables for ENERG & ENERI
    real(8) RMAX,RDFINC,RDINC
    real(8),allocatable::PotNB_riSjS(:,:,:),PotB_riBond(:,:)
    integer(4) NA!,IE
    integer(4),allocatable::NAI(:)
    !------Local Variables--------------------    
    integer(4) I,J,ISA,JSA,IPP,IMOL,JMOL,ITYP,IAT,JAT,KAT
    real(8) DEE,DES,DX,DY,DZ,RR,EFCOR,DEINT,FORCE,DX_IK,DY_IK,DZ_IK,DX_JK,DY_JK,DZ_JK,phi,ENERN,EINTN,EINT_ANGN,EELN,EFURN,ESFN
    ENERN=ENER
    EINTN=EINT
    EINT_angN=EINT_ang
    EELN=EEL
    EFURN=EFUR
    ESFN=ESF
    call EWALD(EFUR,RX,RY,RZ,SX,SY,SZ,Q,SSIN,SCOS,RKV,HBOXL,HBOYL,HBOZL,&
            BOXL,BOYL,BOZL,VOL,ALPHA,AF,RECUT,FQ,COULF,ESF,FKJM,NKV,NCHRG,ICHRG,NSTOT,ID)    
    DE=0.d0
    EINT=0.d0
    EINT_ang=0.d0
    DEE=0.d0
    DES=0.d0
    ! Main loop over all molecules
    do IMOL=1,NOP
        ITYP=MTypeMol(IMOL) ! Molecular type of the molecule
        if (LMoveMType(ITYP)) then
            !First treat pairwise bonds / intramolecular interactions
            do I=AdrAtomMol(IMOL)+1,AdrAtomMol(IMOL)+NSiteMType(ITYP)-1 !Over all atomic pairs in a molecule IMOL
                do J=I+1,AdrAtomMol(IMOL)+NSiteMType(ITYP)
                    DX=SX(I)-SX(J)
                    DY=SY(I)-SY(J)
                    DZ=SZ(I)-SZ(J)
                    call PBC(DX,DY,DZ,HBOXL,HBOYL,HBOZL,BOXL,BOYL,BOZL)
                    RR=sqrt(DX**2+DY**2+DZ**2)
                    IPP=Bond_iSjS(SiteAtom(I),SiteAtom(J))
                    if(IPP.le.0) then ! Intramolecular nonbonded pair
                        if ((IND(1,STypeAtom(I),STypeAtom(J)).ne.-2).and.(RR.le.RCUT)) &
                            DE=DE+ENERG(RR,STypeAtom(I),STypeAtom(J),FORCE, NA,RMAX,RDFINC,PotNB_riSjS,ID)
                        if (RR.le.RECUT) then
                            if ((Q(I)*Q(J)).ne.0.d0) DEE=DEE+Q(I)*Q(J)*COULF*derfc(ALPHA*RR)/RR
                        endif
                    else ! bonded pair (covalent or covalent angle)
                        if ((Q(I)*Q(J)).ne.0.d0) DES=DES+Q(I)*Q(J)*COULF*(derfc(ALPHA*RR)-1.d0)/RR ! Exclude electrostatic interaction in bonded pair
                        if (BTypeBond(IPP).eq.1) then! Intramolecular covalent bonded pair                            
                            if(RR.le.RMN(IPP).or.RR.ge.RMX(IPP)) &
                                write((ID+6),'(a,5i5,3f8.4)')' Bad distance: ',I,J,SiteAtom(I),SiteAtom(J),IPP,RR,RMN(IPP),RMX(IPP)
                            EINT=EINT+ENERI(RR,IPP,FORCE, RMN(IPP),RMX(IPP),PotB_riBond,RDINC,NAI,ID)
                        endif
                    endif
                enddo
            enddo
            ! Second - treat angle bond interactions
            do i=1,num_mt_ang_bond_types(size(num_mt_ang_bond_types,1),ITYP) ! Loop over all covalent angle bonds the molecular type
                IPP=num_mt_ang_bond_types(i,ITYP)        ! Bond
                do j=1,NPTT(IPP) ! Loop over all triplets included in the bond type
                    IAT=mt_ang_bonds(1,j,IPP,ITYP)+AdrAtomMol(IMOL) ! 1st atom
                    JAT=mt_ang_bonds(2,j,IPP,ITYP)+AdrAtomMol(IMOL) ! 2nd atom
                    KAT=mt_ang_bonds(3,j,IPP,ITYP)+AdrAtomMol(IMOL) ! 3rd atom (corner)
                    ! First calculate distances
                    DX_IK=SX(IAT)-SX(KAT)
                    DY_IK=SY(IAT)-SY(KAT)
                    DZ_IK=SZ(IAT)-SZ(KAT)
                    call PBC(DX_IK,DY_IK,DZ_IK,HBOXL,HBOYL,HBOZL,BOXL,BOYL,BOZL)
                    DX_JK=SX(JAT)-SX(KAT)
                    DY_JK=SY(JAT)-SY(KAT)
                    DZ_JK=SZ(JAT)-SZ(KAT)
                    call PBC(DX_JK,DY_JK,DZ_JK,HBOXL,HBOYL,HBOZL,BOXL,BOYL,BOZL)
                    ! Calculate angle
                  phi=dacos((DX_IK*DX_JK+DY_IK*DY_JK+DZ_IK*DZ_JK)/sqrt((DX_IK**2+DY_IK**2+DZ_IK**2)*(DX_JK**2+DY_JK**2+DZ_JK**2)))&
                        *RAD2DEG
                    ! Calculate energy component
                    EINT_ang=EINT_ang+ENERI_ang(phi,IPP,FORCE, RMN(IPP),RMX(IPP),PotB_riBond,RDINC,NAI,ID)
                enddo
            enddo
        endif
        !Than loop over all other molecules - all interactions are non-bonded
        do I=AdrAtomMol(IMOL)+1,AdrAtomMol(IMOL)+NSiteMType(MTypeMol(IMOL)) !Over all atoms in a molecule IMOL
            do JMOL=IMOL+1,NOP ! when IMOL=NOP, starting value in this loop will be NOP+1, and the cycle should not run
                do J=AdrAtomMol(JMOL)+1,AdrAtomMol(JMOL)+NSiteMType(MTypeMol(JMOL)) !Over all atoms in a molecule JMOL
                    DX=SX(I)-SX(J)
                    DY=SY(I)-SY(J)
                    DZ=SZ(I)-SZ(J)
                    call PBC(DX,DY,DZ,HBOXL,HBOYL,HBOZL,BOXL,BOYL,BOZL)
                    RR=sqrt(DX**2+DY**2+DZ**2)
                    if (LMoveMType(MTypeMol(IMOL)).or.(LMoveMType(MTypeMol(JMOL)))) then
                        if (RR.le.RCUT) DE=DE+ENERG(RR,STypeAtom(I),STypeAtom(J),FORCE,NA,RMAX,RDFINC,PotNB_riSjS,ID)
                    endif
                    if (RR.le.RECUT) then
                        if ((Q(I)*Q(J)).ne.0.d0) DEE=DEE+Q(I)*Q(J)*COULF*derfc(ALPHA*RR)/RR
                    endif
                enddo
            enddo
        enddo
    enddo
    EEL=ESF+EFUR+DEE+DES
    ENER=EEL+DE+EINT+EINT_ang
    EFCOR=ESF+EFUR+DES
    if(dabs(ENERN-ENER).gt.0.01) then
        write((ID+6),'(2(a,f14.5))') 'ENERGY: Error in energy: was ',ENERN*FKJM,' now ',ENER*FKJM
        write((ID+6),'(a15,6f14.6)') 'Before Energy:',ENERN*FKJM,EELN*FKJM,EINTN*FKJM,EINT_angN*FKJM,EFURN*FKJM,ESFN*FKJM
        write((ID+6),'(a15,6f14.6)') 'After Energy:',ENER*FKJM,EEL*FKJM,EINT*FKJM,EINT_ang*FKJM,EFUR*FKJM,ESF*FKJM
    endif

    if(IPRINT.ge.7)then
        write((ID+6),*)'*** ENERGY components ***'
        write((ID+6),*)' total:         ',ENER*FKJM
        write((ID+6),*)' short range:   ',DE*FKJM
        write((ID+6),*)' electrostatic: ',EEL*FKJM
        write((ID+6),*)' bond:          ',EINT*FKJM
        write((ID+6),*)' angle bond:    ',EINT_ang*FKJM
        write((ID+6),*)' real Ew:       ',DEE*FKJM
        write((ID+6),*)' Fourier:       ',EFUR*FKJM
        write((ID+6),*)' self-mol:      ',DES*FKJM
        write((ID+6),*)' Ewald self-int:',ESF*FKJM
        write((ID+6),*)' Ewald self-mol:',(ESF+DES)*FKJM
        write((ID+6),*)' tot Ew.corr:   ',EFCOR*FKJM
        
        do I=1,NSTOT
            write((ID+6),'(I5,f8.2,4f10.4)')I,Q(I),SX(I),SY(I),SZ(I)
        end do
    end if
        
    end subroutine ENERGY

!===================== Correl ========================================
!
subroutine Correl(EFUR,RX,RY,RZ,SX,SY,SZ,Q,SSIN,SCOS,RKV,EEL,VIR,VIRI,EINT,EINT_ang,&
    		RMAX,RMIN,RCUT,RDFINC,PotNB_riSjS,RMN,RMX,PotB_riBond,RDINC,&
		ESF,ENER,CORS,CORP,ENERS,EELS,NKV,NCHRG,&
		HBOXL,HBOYL,HBOZL,BOXL,BOYL,BOZL,VOL,ALPHA,AF,RECUT,FQ,COULF,FKJM,&
		ICHRG,NA,STypeAtom,IND,INDI,&
		IndexGlobNB_riSjS,IndexGlobB_riBond, SiteAtom,Bond_iSjS,NAI,NAV,NSTOT,&
		LMoveMType,LIMC,npairsrdf,ID,NII,NII_ang,BTypeBond,&
                NOP,AdrAtomMol,NSiteMType,MTypeMol,num_mt_ang_bond_types,MT_ang_bonds,NPTT,StateNB_iSjS,StateB_iBond)
    implicit none
!------Global Variables--------------------
    integer(4), allocatable::AdrAtomMol(:),NSiteMType(:),MTypeMol(:),num_mt_ang_bond_types(:,:),MT_ang_bonds(:,:,:,:),NPTT(:),&
                            STypeAtom(:),SiteAtom(:),ICHRG(:),IND(:,:,:),INDI(:,:),IndexGlobNB_riSjS(:,:,:),IndexGlobB_riBond(:,:),&
                            Bond_iSjS(:,:),NAI(:),&
                            BTypeBond(:), StateNB_iSjS(:,:),StateB_iBond(:)
    integer(4) NKV,NCHRG,NA,NAV,NSTOT,ID,NOP,NII,NII_ang
    real(8) EFUR,EEL,VIR,VIRI,EINT,EINT_ang,RMAX,RMIN,RCUT,RDFINC,RDINC,ESF,ENER,&
            HBOXL,HBOYL,HBOZL,BOXL,BOYL,BOZL,VOL,ALPHA,AF,RECUT,FQ,COULF,FKJM,ENERS,EELS
    real(8),allocatable::RX(:),RY(:),RZ(:),RKV(:),SSIN(:),SCOS(:),PotNB_riSjS(:,:,:),&
                    PotB_riBond(:,:),RMN(:),RMX(:),SX(:),SY(:),SZ(:),Q(:)   
    integer(8), allocatable:: npairsrdf(:,:),CORS(:),CORP(:,:)
    logical LMoveMType(:),LIMC
    !------Local Variables----------------------
    integer(8), allocatable::  CCOR(:)
    integer(8) CCC
    integer(4) IABN,I,IT,IST,IMOL,J,JT,JST,JMOL,NR,ISA,JSA,IPP,NAM,NR1,IPI,ITYP,IAT,JAT,KAT,Nphi
    real(8) DE,DEE,DRR,DX,DY,DZ,RR,R1,R2,RD,DDE,ETT,FORCE,ENERN,EINTN,EINT_ANGN,EELN,EFURN,ESFN,&
            DX_IK,DY_IK,DZ_IK,DX_JK,DY_JK,DZ_JK,phi 
    allocate(CCOR(size(CORS)))
    CCOR(:)=0
    ENERN=ENER
    EINTN=EINT
    EINT_angN=EINT_ang
    EELN=EEL
    EFURN=EFUR
    ESFN=ESF
    call EWALD(EFUR,RX,RY,RZ,SX,SY,SZ,Q,SSIN,SCOS,RKV,HBOXL,HBOYL,HBOZL,&
		BOXL,BOYL,BOZL,VOL,ALPHA,AF,RECUT,FQ,COULF,ESF,FKJM,NKV,NCHRG,ICHRG,NSTOT,ID)
    EEL=EFUR+ESF
    DE=0.d0
    DEE=0.d0
    VIR=0.d0
    VIRI=0.d0
    EINT=0.d0
    EINT_ang=0.d0
    IABN=0
    CCOR(:)=0
   ! Main loop over all molecules
    do IMOL=1,NOP
        !First treat intramolecular interactions - bonds
        ITYP=MTypeMol(IMOL)
        do I=AdrAtomMol(IMOL)+1,AdrAtomMol(IMOL)+NSiteMType(ITYP)-1 !Over all atomic pairs in a molecule IMOL
            IST=STypeAtom(I)
            do J=I+1,AdrAtomMol(IMOL)+NSiteMType(ITYP)
                JST=STypeAtom(J)
                DX=SX(I)-SX(J)
                DY=SY(I)-SY(J)
                DZ=SZ(I)-SZ(J)
                call PBC(DX,DY,DZ,HBOXL,HBOYL,HBOZL,BOXL,BOYL,BOZL)
                RR=sqrt(DX**2+DY**2+DZ**2)
                IPP=Bond_iSjS(SiteAtom(I),SiteAtom(J))
                if(IPP.le.0) then ! Intramolecular nonbonded pair
                    if (LMoveMType(ITYP))then
                        if(RR.le.RCUT)then
                            NR=RR*NA/RMAX+1
                            if(IND(NR,IST,JST).eq.0)then
                                write((ID+6),'(a,i4,f12.4)')  ' !!! Wrong configuration:',NR,RR
                                write((ID+6),'(I5,3f10.3)')I,SX(I),SY(I),SZ(I)
                                write((ID+6),'(I5,3f10.3)')J,SX(J),SY(J),SZ(J)
                                IABN=IABN+1
                            elseif (IND(NR,IST,JST).le.-2) then
                                IABN=IABN+1
                            end if   !  if(IND...
                            DE=DE+ENERG(RR,IST,JST,FORCE,NA,RMAX,RDFINC,PotNB_riSjS,ID)
                            VIR=VIR+RR*FORCE
                            if (Btest(StateNB_iSjS(IST,JST),4)) &
                                    CCOR(IndexGlobNB_riSjS(NR,IST,JST))=CCOR(IndexGlobNB_riSjS(NR,IST,JST))+1
                        else ! RR>RCUT
                            npairsrdf(IST,JST)=npairsrdf(IST,JST)+1
                        end if
                    end if
                    if(RR.le.RECUT) then
                         if ((Q(I)*Q(J)).ne.0.d0) DEE=DEE+Q(I)*Q(J)*COULF*derfc(ALPHA*RR)/RR
                    endif
                else
                    if ((Q(I)*Q(J)).ne.0.d0) DEE=DEE+Q(I)*Q(J)*COULF*(derfc(ALPHA*RR)-1.d0)/RR ! Discount electrostatic interaction  if pair is bonded
                    if (BTypeBond(IPP).eq.1) then! Intramolecular bonded pair with covalent bond                        
                        NAM=NAI(IPP)
                        R1=RMN(IPP)
                        R2=RMX(IPP)
                        if(RR.lt.R1.or.RR.gt.R2)then ! RR - out of allowed bond range
                                write((ID+6),'(a,i5,f10.4)')' Wrong intramolecular distance in ',IMOL,RR
                                write((ID+6),'(I5,3f10.3)')I,SX(I),SY(I),SZ(I)
                                write((ID+6),'(I5,3f10.3)')J,SX(J),SY(J),SZ(J)
                                stop
                        else ! RR in allowed bond range
                            NR1=(RR-R1)*NAM/(R2-R1)+1.
                        end if
                        EINT=EINT+ENERI(RR,IPP,FORCE,RMN(IPP),RMX(IPP),PotB_riBond,RDINC,NAI,ID)
                        VIRI=VIRI+RR*FORCE
                        if (BTest(StateB_iBond(IPP),4)) CCOR(IndexGlobB_riBond(NR1,IPP))=CCOR(IndexGlobB_riBond(NR1,IPP))+1
                    endif
	    	endif
            enddo
        enddo
    ! Second - treat covalent angle bond interactions
        do i=1,num_mt_ang_bond_types(size(num_mt_ang_bond_types,1),ITYP) ! Loop over all covalent angle bond types in the molecule
            IPP=num_mt_ang_bond_types(i,ITYP)        ! Bond type
            do j=1,NPTT(IPP) ! Loop over all triplets included in the bond type
                IAT=mt_ang_bonds(1,j,IPP,ITYP)+AdrAtomMol(IMOL) ! 1st atom
                JAT=mt_ang_bonds(2,j,IPP,ITYP)+AdrAtomMol(IMOL) ! 2nd atom
                KAT=mt_ang_bonds(3,j,IPP,ITYP)+AdrAtomMol(IMOL) ! 3rd atom (corner)
                ! First calculate distances
                DX_IK=SX(IAT)-SX(KAT)
                DY_IK=SY(IAT)-SY(KAT)
                DZ_IK=SZ(IAT)-SZ(KAT)
                call PBC(DX_IK,DY_IK,DZ_IK,HBOXL,HBOYL,HBOZL,BOXL,BOYL,BOZL)
                DX_JK=SX(JAT)-SX(KAT)
                DY_JK=SY(JAT)-SY(KAT)
                DZ_JK=SZ(JAT)-SZ(KAT)
                call PBC(DX_JK,DY_JK,DZ_JK,HBOXL,HBOYL,HBOZL,BOXL,BOYL,BOZL)
                ! Calculate angle
               phi=dacos((DX_IK*DX_JK+DY_IK*DY_JK+DZ_IK*DZ_JK)/sqrt((DX_IK**2+DY_IK**2+DZ_IK**2)*(DX_JK**2+DY_JK**2+DZ_JK**2)))*&
                        RAD2DEG
                ! Check phi and find related point in global potential array
                NAM=NAI(IPP)
                R1=RMN(IPP) !Minimal value of angle
                R2=RMX(IPP) !Maximal value of angle
                if(phi.lt.R1.or.phi.gt.R2)then ! phi - out of allowed bond angle range
                        write((ID+6),'(a,i5,f10.4)')' Wrong intramolecular bond angle in ',IMOL,phi
                        write((ID+6),'(I5,3f10.3)')IAT,SX(IAT),SY(IAT),SZ(IAT)
                        write((ID+6),'(I5,3f10.3)')JAT,SX(JAT),SY(JAT),SZ(JAT)
                        write((ID+6),'(I5,3f10.3)')KAT,SX(KAT),SY(KAT),SZ(KAT)
                        stop
                else ! RR in allowed bond range
                    Nphi=(phi-R1)*NAM/(R2-R1)+1.
                end if
                if (BTest(StateB_iBond(IPP),4)) CCOR(IndexGlobB_riBond(Nphi,IPP))=CCOR(IndexGlobB_riBond(Nphi,IPP))+1
                ! Calculate energy component
!DEBUG                    write(ID+6,*) DX_IK,DY_IK,DZ_IK,DX_JK,DY_JK,DZ_JK,phi
                EINT_ang=EINT_ang+ENERI_ang(phi,IPP,FORCE,RMN(IPP),RMX(IPP),PotB_riBond,RDINC,NAI,ID)
                VIRI=VIRI+RR*FORCE
            enddo
        enddo
        !Than loop over all other molecules - all interactions are non-bonded
        do I=AdrAtomMol(IMOL)+1,AdrAtomMol(IMOL)+NSiteMType(MTypeMol(IMOL)) !Over all atoms in a molecule IMOL
            IST=STypeAtom(I)
            do JMOL=IMOL+1,NOP ! when IMOL=NOP, starting value in this loop will be NOP+1, and the cycle should not run
                do J=AdrAtomMol(JMOL)+1,AdrAtomMol(JMOL)+NSiteMType(MTypeMol(JMOL)) !Over all atoms in a molecule JMOL
                    JST=STypeAtom(J)
                    DX=SX(I)-SX(J)
                    DY=SY(I)-SY(J)
                    DZ=SZ(I)-SZ(J)
                    call PBC(DX,DY,DZ,HBOXL,HBOYL,HBOZL,BOXL,BOYL,BOZL)
                    RR=sqrt(DX**2+DY**2+DZ**2)                    
                    if (LMoveMType(MTypeMol(IMOL)).or.LMoveMType(MTypeMol(JMOL)))then
                        if(RR.le.RCUT)then
                            NR=RR*NA/RMAX+1
                            if(IND(NR,IST,JST).eq.0)then
                                write((ID+6),'(a,i4,f12.4)')  ' !!! Wrong configuration:',NR,RR
                                write((ID+6),'(I5,3f10.3)')I,SX(I),SY(I),SZ(I)
                                write((ID+6),'(I5,3f10.3)')J,SX(J),SY(J),SZ(J)
                                IABN=IABN+1
                            elseif (IND(NR,IST,JST).le.-2) then
                                IABN=IABN+1
                            end if   !  if(IND...
                            DE=DE+ENERG(RR,IST,JST,FORCE,NA,RMAX,RDFINC,PotNB_riSjS,ID)
                            VIR=VIR+RR*FORCE
                            CCOR(IndexGlobNB_riSjS(NR,IST,JST))=CCOR(IndexGlobNB_riSjS(NR,IST,JST))+1
                        else ! RR>RCUT
                            npairsrdf(IST,JST)=npairsrdf(IST,JST)+1
                        end if
                    end if
                    if(RR.le.RECUT) then
                        if ((Q(I)*Q(J)).ne.0.d0) DEE=DEE+Q(I)*Q(J)*COULF*derfc(ALPHA*RR)/RR
                    endif
                enddo
            enddo
        enddo
    enddo
    EEL=ESF+EFUR+DEE
    ENER=EEL+DE+EINT+EINT_ang
!  correlators           CCOR(I) = S_alpha   (alpha = I)
    do I=1,size(CORS) ! По всем точкам глобального п-ала (интер и интра)
        CCC=CCOR(I)
	if(CCC.ne.0.d0)then ! Если число пар ненулевое
            CORS(I)=CORS(I)+CCC !
            if(LIMC)then ! Если включен инверсный режим
                do J=I,size(CORS) ! 
                    CORP(I,J)=CORP(I,J)+CCC*CCOR(J)     !  S_alpha * S_gamma
	    	end do
            end if
	end if
    end do
    ENERS=ENERS+ENER
    EELS=EELS+EEL
    NAV=NAV+1
    deallocate(CCOR)
    if(dabs(ENERN-ENER).gt.0.01) then 
        write((ID+6),'(2(a,f14.5))') 'Correl: Error in energy: was ',ENERN*FKJM,' now ',ENER*FKJM
        write((ID+6),'(a15,4f14.6)') '                ENER         EEL           EINTN         EINT_ang      '
       write((ID+6),'(a15,6f14.6)') 'Before correl:',ENERN*FKJM,EELN*FKJM,EINTN*FKJM,EINT_angN*FKJM,EFURN*FKJM,ESFN*FKJM
        write((ID+6),'(a15,6f14.6)') 'After correl:',ENER*FKJM,EEL*FKJM,EINT*FKJM,EINT_ang*FKJM,EFUR*FKJM,ESF*FKJM

    endif
end subroutine Correl

end !Module
